######################################################################
## MAKEFILE  Makefile for WindowFrame
## Chapter 1 example programs
## MSC 7.0
######################################################################
## Usage Notes:
## To build dt.lib:
##     nmake /fmsc70.mak
##          or
##     nmake /fmsc70.mak all
##
## To clean intermediate *.obj files:
##     nmake /fmsc70.mak clean
######################################################################


######################################################################
CCDEBUG = /Od /Zi
MODEL = S       # C small memory model
USR_INCLUDE_DIR =   ..\include
USR_LIB_DIR     =   ..\lib
##I = c:\c700\include   ## Environment variables used instead

CC = cl
CFLAGS = $(CCDEBUG) /A$(MODEL) /Gs /I$(USR_INCLUDE_DIR)
USRLIB=wf$(MODEL).lib
######################################################################
## Meaning of CFLAGS for MSC 7.0 Compiler
## /AS      small memory model
## /Gs      turn off run-time stack checking
## /Od      turn off all optimizations (use with /Zi for debugging)
## /Zi      include symbolic debug information in obj files
## /Ipath   dir indicates additional path to search for #include
######################################################################

######################################################################
## Meaning of LFLAGS for MSC 7.0 Linker
## /CO      include symbolic debug information in exe files
######################################################################


# Include standard definitions
!include "..\defs_msc.mak"


############################################################
## Default target
############################################################
default: paint.exe

all:                \
    template.exe    \
    tmpl1.exe       \
    tmpl2.exe       \
    menu2.exe       \
    paint.exe       \
    paint2.exe      \
    menu3.exe       \
    mouse1.exe
############################################################

# Expand inline methods for library
.cpp.obj :
    $(CC) $(CCOPTS) $<

############################################################
## TEMPLATE
############################################################
template.exe: template.obj $(L)wf$(MODEL).lib
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc $@
#    rc $<
############################################################


############################################################
## TMPL1
############################################################
tmpl1.exe: tmpl1.obj $(L)wf$(MODEL).lib
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc $@
############################################################


############################################################
## TMPL2
############################################################
tmpl2.exe: tmpl2.obj $(L)wf$(MODEL).lib
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc $@
############################################################


############################################################
## MENU2
############################################################
menu2.exe: menu2.obj $(L)wf$(MODEL).lib menu1.res
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc -K menu1.res $@
############################################################


############################################################
## MENU3
############################################################
menu3.exe: menu3.obj $(L)wf$(MODEL).lib menu1.res
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc -K menu1.res $@
############################################################


############################################################
## PAINT
############################################################
paint.exe: paint.obj $(L)wf$(MODEL).lib menu1.res
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc -K menu1.res $@
############################################################


############################################################
## PAINT2
############################################################
paint2.exe: paint2.obj $(L)wf$(MODEL).lib menu1.res
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc -K menu1.res $@
############################################################



############################################################
## MOUSE1
############################################################
mouse1.exe: mouse1.obj $(L)wf$(MODEL).lib menu2.res
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc -K menu2.res $@
############################################################


clean:
    del *.obj
    del *.map
    del *.sym

