//
// DTMAIN.CPP   Main window callback functions for dialog test program.
//
// Created:     Bruce Clift  8/15/91
//

#include <windows.h>

#include "window.hpp"
#include "handler.hpp"
#include "paint.hpp"
#include "modal.hpp"

#include "dtmain.hpp"


//////////////////////////////
// CALLBACK FUNCTION TABLES //
//////////////////////////////

static CB MsgCbList[] = {
    { WM_INITMENU,  (CBPROC) &WinDlgTst::wmInitMenu },
    { 0, NULL }
};

static CB CmdCbList[] = {
    { IDM_MODELESS, (CBPROC) &WinDlgTst::idmModeless },
    { IDM_ABOUT,    (CBPROC) &WinDlgTst::idmAbout },
    { 0, NULL }
};

Handler WinDlgTst::aHandler(MsgCbList, CmdCbList);


////////////////////////
// CALLBACK FUNCTIONS //
////////////////////////

//
// wmInitMenu un/checks the Modeless dialog box menu item, depending on its
// current state.
//
LRESULT
WinDlgTst::wmInitMenu(UINT, WPARAM wParam, LPARAM)
{
    // Un/check Modeless menu item
    CheckMenuItem(wParam, IDM_MODELESS,
        MF_BYCOMMAND | (modelessDlg ? MF_CHECKED : MF_UNCHECKED) );
    return 0L;
}

//
// idmModeless runs the Modeless Dialog Box.
//
LRESULT
WinDlgTst::idmModeless(UINT, WPARAM, LPARAM)
{
    // Toggle the modeless dialog box on and off
    if (modelessDlg)
            modelessDlg.destroy();
    else    modelessDlg.create("Modeless", hwnd());
    aStatusBar.invalidate();
    update();
    return 0L;
}

//
// idmAbout runs the About Dialog Box.
//
LRESULT
WinDlgTst::idmAbout(UINT, WPARAM, LPARAM)
{
    // Update the status bar area (paint immediately)
    iHelp = (modelessDlg ? IDS_BOTH : IDS_ABOUT);
    aStatusBar.invalidate();
    update();

    // Construct and run a dialog box object
    Modal about;
    about.run("About", hwnd());

    aStatusBar.invalidate();
    return 0L;
}


//
// This method returns a string table index used to get a status bar help
// string.
//
int
WinDlgTst::getHelpIndex(void)
{
    int i = iHelp;
    if (!i)
        i = (modelessDlg ? IDS_MDLS : IDS_NOMDLS);
    iHelp = 0;
    return i;
}
