#include <windows.h>
#include "app.hpp"

#include "window.hpp"
#include "handler.hpp"
#include "paint.hpp"
#include "rect.hpp"
#include "filedlg.hpp"
#include "file.hpp"
#include "fdlg.h"

////////////////////////////////////////////////////////////
// class AppWin (interface)
////////////////////////////////////////////////////////////
class AppWin : public Window
{
    static Handler aHandler;
public:
    Handler *getHandler(void) { return &aHandler; }
    long wmClose(UINT, WPARAM, LPARAM);
    long idmOpen(UINT, WPARAM, LPARAM);
    long idmSave(UINT, WPARAM, LPARAM);
    long idmExit(UINT, WPARAM, LPARAM);
};

////////////////////////////////////////////////////////////
// class AppWin (implementation)
////////////////////////////////////////////////////////////
long AppWin::idmOpen(UINT, WPARAM, LPARAM)
{
    File aFile;

    FileOpenDialog aFOD(&aFile, ".CPP", "*.CPP", 0);
    if (!aFOD.run("FileOpen", hwnd()))
        return 0L;
    MessageBox(hwnd(),
               aFOD.getFileName(),
               "File Selected:",
               MB_OK);
    return 0L;
}

long AppWin::idmSave(UINT, WPARAM, LPARAM)
{
    File aFile;

    FileSaveDialog aFSD(&aFile, ".CPP", "*.CPP");
    if (!aFSD.run("FileSave", hwnd()))
        return 0L;
    if (aFSD.getStatus() == 1)
    {
        MessageBox(hwnd(),
                   "Already exists",        // message
                   aFSD.getFileName(),      // title
                   MB_OK);
    }
    else
    {
        MessageBox(hwnd(),
                   "New file",              // message
                   aFSD.getFileName(),      // title
                   MB_OK);
    }
    return 0L;
}

long AppWin::idmExit(UINT, WPARAM, LPARAM)
{
    return wmClose(WM_CLOSE, 0, 0L);
}

long AppWin::wmClose(UINT, WPARAM, LPARAM)
{
    if (IDOK == MessageBox(hwnd(),
                           "Terminate program?",
                           "Exit TestDialog",
                           MB_OKCANCEL | MB_ICONQUESTION))
    {
        destroy();
    }
    return 0L;
}

////////////////////////////////////////////////////////////
// class AppWin (callbacks)
////////////////////////////////////////////////////////////
static CB MsgCBT[] = {
    { WM_CLOSE, (CBPROC) AppWin::wmClose },
    { 0,        NULL }
};

static CB CmdCBT[] = {
    { IDM_OPEN, (CBPROC) AppWin::idmOpen },
    { IDM_SAVE, (CBPROC) AppWin::idmSave },
    { IDM_EXIT, (CBPROC) AppWin::idmExit },
    { 0,   NULL }
};

Handler AppWin::aHandler(MsgCBT, CmdCBT);

////////////////////////////////////////////////////////////
// WinMain
////////////////////////////////////////////////////////////
int PASCAL WinMain(HANDLE hInstance,
                   HANDLE hPrevInstance, LPSTR, int)
{
    AppWin aWindow;
    aWindow.setCaption("FileOpenDialog and FileSaveDialog");
    aWindow.setMenuName(ID_FILEMENU);

    App anApp(hInstance, hPrevInstance);
    return anApp.run(aWindow, "FileDlg");
}

