//
// HELLO.CPP    Window Frame "Hello, World" application.
//
// Created:     Bruce Clift  6/25/91
//

#include <windows.h>

#include "window.hpp"
#include "handler.hpp"
#include "paint.hpp"
#include "rect.hpp"
#include "app.hpp"


///////////////////////
// MAIN WINDOW CLASS //
///////////////////////

class HelloWin : public Window {
private:
    static Handler aHandler;        // Use Handler object for messages

public:
    // Data member access
    Handler *getHandler(void) { return &aHandler; }

    // Callback methods
    LRESULT wmPaint(UINT, WPARAM, LPARAM);

};

// Callback method table
static CB MsgCbList[] = {
    { WM_PAINT, (CBPROC) &HelloWin::wmPaint },
    { 0, NULL }
};

// Handler constructor builds message callback list
Handler HelloWin::aHandler(MsgCbList);


//
// wmPaint writes the text "Hello, World" in the main window.
//
LRESULT
HelloWin::wmPaint(UINT, WPARAM, LPARAM)
{
    Paint p(hwnd());    // Construct paint object (derived from PAINTSTRUCT)

    // Draw the text in the middle of the window's client area
    Rect aRect(hwnd());
    DrawText(p, "Hello, world", -1, &aRect,
        DT_SINGLELINE | DT_CENTER | DT_VCENTER);
    return 0L;
}


//////////////////////
// WINMAIN FUNCTION //
//////////////////////

//
// The WinMain entry procedure simply creates the derived application object,
// passing its parameters to the constructor, and the runs the application.
//
int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR, int)
{
    HelloWin aWindow;
    aWindow.setCaption("WindowFrame Hello");

    App anApp(hInstance, hPrevInstance);
    return anApp.run(aWindow, "Hello");
}
