######################################################################
## MAKEFILE  Makefile for WindowFrame
## Chapter 2 example programs
## MSC 7.0
######################################################################
## Usage Notes:
## To build dt.lib:
##     nmake /fmsc70.mak
##          or
##     nmake /fmsc70.mak all
##
## To clean intermediate *.obj files:
##     nmake /fmsc70.mak clean
######################################################################


######################################################################
CCDEBUG = /Od /Zi
MODEL = S       # C small memory model
USR_INCLUDE_DIR =   ..\include
USR_LIB_DIR     =   ..\lib
##I = c:\c700\include   ## Environment variables used instead

CC = cl
CFLAGS = $(CCDEBUG) /A$(MODEL) /Gs /I$(USR_INCLUDE_DIR)
USRLIB=wf$(MODEL).lib
######################################################################
## Meaning of CFLAGS for MSC 7.0 Compiler
## /AS      small memory model
## /Gs      turn off run-time stack checking
## /Od      turn off all optimizations (use with /Zi for debugging)
## /Zi      include symbolic debug information in obj files
## /Ipath   dir indicates additional path to search for #include
######################################################################

######################################################################
## Meaning of LFLAGS for MSC 7.0 Linker
## /CO      include symbolic debug information in exe files
######################################################################


# Include standard definitions
!include "..\defs_msc.mak"


############################################################
## Default target
############################################################
default: dlgtst.exe

all:                \
    hello.exe       \
    fdlg.exe        \
    dlgtst.exe      \
    template.exe    

#
# WINDOW FRAME TEMPLATE
#

## template.exe: template.obj $(L)wf$(MODEL).lib
##    $(LNK) $(LNKOPTS) template.obj,$*,nul,$(WFLIBS),$(L)wf
##    rc $<
############################################################
## TEMPLATE
############################################################
template.exe: template.obj $(L)wf$(MODEL).lib
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc $@
#    rc $<
############################################################


############################################################
## HELLO
############################################################
#
# HELLO WORLD PROGRAM
#

hello.exe: hello.obj $(L)wf$(MODEL).lib
##    $(LNK) $(LNKOPTS) hello.obj,$*,nul,$(WFLIBS),$(L)wf
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc $@
##    rc $<


############################################################
## FDLG
############################################################
#
# WINDOW FRAME FILEDIALOG
#

fdlg.exe: fdlg.obj $(L)wf$(MODEL).lib fdlg.res
##    $(LNK) $(LNKOPTS) fdlg.obj,$*,nul,$(WFLIBS),$(L)wf
    $(LNK) $(LNKOPTS) @<<link.rsp
    $*.obj
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc fdlg.res
##    rc $@
##    rc fdlg.res


############################################################
## DLGTST
############################################################
#
# DIALOG TEST PROGRAM
#

DTOBJS=dlgtst.obj dtmain.obj

dlgtst.exe: $(DTOBJS) $(L)wf$(MODEL).lib dlgtst.res
##    $(LNK) $(LNKOPTS) $(DTOBJS),$*,nul,$(WFLIBS),$(L)wf
    $(LNK) $(LNKOPTS) @<<link.rsp
    $(DTOBJS)
    $*
    nul,
    $(WFLIBS) +
    $(CCDIR)\lib\$(MODEL)libcew +
    $(CCDIR)\lib\libw
    $(L)wf
<<KEEP
    rc dlgtst.res
##    rc dlgtst.res

############################################################



clean:
    del *.obj
    del *.res
    del *.map
    del *.sym
    del *.rsp

