//
// MPAINT.CPP   Mini-paint demonstration program.
//
// Created      Bruce Clift  8/24/92
//

#include <windows.h>
//#include <mem.h>
#include "wfdefs.hpp"
#if (0 MSC(+1))         // Microsoft C++
#include <memory.h>
#elif (0 BCC(+1))       // Borland C++
#include <mem.h>
#endif

#include "ownerdra.hpp"
#include "brush.hpp"
#include "caret.hpp"
#include "font.hpp"
#include "rect.hpp"
#include "window.hpp"
#include "handler.hpp"
#include "bitmap.hpp"
#include "point.hpp"
#include "devconte.hpp"
#include "textmetr.hpp"
#include "pen.hpp"
#include "paint.hpp"
#include "region.hpp"
#include "app.hpp"

#include "mpaint.h"

#if (0 BCC(+1)) // Borland C++
#define min(x,y) ((x) < (y) ? (x) : (y))
#define max(x,y) ((x) > (y) ? (x) : (y))
#endif


/////////////////////////
// TOOL BUTTON CLASSES //
/////////////////////////

class ColorButton : public OwnerDrawButton {
private:
    DWORD dwColor;

public:
    ColorButton(DWORD dw) : dwColor(dw) {}
    DWORD getColor(void) { return dwColor; }

    void drawEntire(LPDRAWITEMSTRUCT lpDIS);
};

void
ColorButton::drawEntire(LPDRAWITEMSTRUCT lpDIS)
{
    Brush aBrush(dwColor);
    HBRUSH hbrOld = aBrush.select(lpDIS->hDC);
    LPRECT lpR = &lpDIS->rcItem;
    Rectangle (lpDIS->hDC, lpR->left, lpR->top, lpR->right, lpR->bottom);
    DeleteObject (SelectObject (lpDIS->hDC, hbrOld));
}


///////////////////////
// MAIN WINDOW CLASS //
///////////////////////

class MiniPaintWin : public Window {
private:
    static Handler mainHandler;
    static DWORD colors[MAXCOLORS];

    Caret aCaret;                       // Caret for typing
    int xCaret, yCaret;                 // Current position of caret
    int cxChar, cyChar;                 // Size of average char
    Font aFont;                         // Font for typing text

    Rect drawRect;                      // Area for drawing
    OwnerDrawButton *tools[MAXTOOLS];   // Tool buttons
    ColorButton *palette[MAXCOLORS];    // Color buttons
    WORD wTool, wColor;                 // Current tool and color
    Point ptStart, ptLast;      // Starting and current mouse position

public:
    // Constructors and destructors manage instance variables
    MiniPaintWin();
    ~MiniPaintWin();

    // Data member access
    Handler *getHandler(void) { return &mainHandler; }
    char *getName(void) { return "MPaint"; }

    // Callback methods
    LRESULT idcTool(UINT, WPARAM, LPARAM);
    LRESULT idcColor(UINT, WPARAM, LPARAM);

    LRESULT wmCreate(UINT, WPARAM, LPARAM);
    LRESULT wmSize(UINT, WPARAM, LPARAM);
    LRESULT wmDrawItem(UINT, WPARAM, LPARAM);
    LRESULT wmLButtonDown(UINT, WPARAM, LPARAM);
    LRESULT wmMouseMove(UINT, WPARAM, LPARAM);
    LRESULT wmLButtonUp(UINT, WPARAM, LPARAM);
    LRESULT wmSetFocus(UINT, WPARAM, LPARAM);
    LRESULT wmKillFocus(UINT, WPARAM, LPARAM);
    LRESULT wmChar(UINT, WPARAM, LPARAM);
    LRESULT wmPaint(UINT, WPARAM, LPARAM);

    // Utility method for drawing current shape
    void drawShape(HDC hdc, POINT ptFrom, POINT ptTo);

};

// Callback method tables
static CB MsgCbList[] = {
    { WM_CREATE,        (CBPROC) &MiniPaintWin::wmCreate },
    { WM_SIZE,          (CBPROC) &MiniPaintWin::wmSize },
    { WM_DRAWITEM,      (CBPROC) &MiniPaintWin::wmDrawItem },
    { WM_LBUTTONDOWN,   (CBPROC) &MiniPaintWin::wmLButtonDown },
    { WM_MOUSEMOVE,     (CBPROC) &MiniPaintWin::wmMouseMove },
    { WM_LBUTTONUP,     (CBPROC) &MiniPaintWin::wmLButtonUp },
    { WM_SETFOCUS,      (CBPROC) &MiniPaintWin::wmSetFocus },
    { WM_KILLFOCUS,     (CBPROC) &MiniPaintWin::wmKillFocus },
    { WM_CHAR,          (CBPROC) &MiniPaintWin::wmChar },
    { WM_PAINT,         (CBPROC) &MiniPaintWin::wmPaint },
    { 0, NULL }
};

static CB CmdCbList[] = {
    { IDC_PEN,          (CBPROC) &MiniPaintWin::idcTool },
    { IDC_ERASER,       (CBPROC) &MiniPaintWin::idcTool },
    { IDC_CIRCLE,       (CBPROC) &MiniPaintWin::idcTool },
    { IDC_TEXT,         (CBPROC) &MiniPaintWin::idcTool },
    { IDC_RECT,         (CBPROC) &MiniPaintWin::idcTool },
    { IDC_ROUND,        (CBPROC) &MiniPaintWin::idcTool },
    { IDC_BLACK,        (CBPROC) &MiniPaintWin::idcColor },
    { IDC_RED,          (CBPROC) &MiniPaintWin::idcColor },
    { IDC_GREEN,        (CBPROC) &MiniPaintWin::idcColor },
    { IDC_YELLOW,       (CBPROC) &MiniPaintWin::idcColor },
    { IDC_BLUE,         (CBPROC) &MiniPaintWin::idcColor },
    { IDC_MAGENTA,      (CBPROC) &MiniPaintWin::idcColor },
    { IDC_CYAN,         (CBPROC) &MiniPaintWin::idcColor },
    { IDC_WHITE,        (CBPROC) &MiniPaintWin::idcColor },
    { 0, NULL }
};

// Handler constructor builds message callback list
Handler MiniPaintWin::mainHandler(MsgCbList, CmdCbList);


//
// Array of colors in the palette
//
DWORD MiniPaintWin::colors[] = {
    RGB(  0,   0, 0), RGB(  0,   0, 255),
    RGB(  0, 255, 0), RGB(  0, 255, 255),
    RGB(255,   0, 0), RGB(255,   0, 255),
    RGB(255, 255, 0), RGB(255, 255, 255),
};


//
// The constructors clears the button pointer arrays and initializes
// other data members.
//
MiniPaintWin::MiniPaintWin()
    : wTool(IDC_PEN), wColor(IDC_BLACK), ptStart(0, 0)
{
    memset(tools, 0, sizeof(tools));
    memset(palette, 0, sizeof(palette));
}

//
// The destructor deletes all tool and color buttons, as well as the
// tool buttons' bitmaps.
//
MiniPaintWin::~MiniPaintWin()
{
    int i;
    for (i=0; i < MAXTOOLS; i++) {
        if (tools[i] == NULL)
            break;
        delete tools[i]->getBitmap1();
        delete tools[i];
    }
    for (i=0; i < MAXCOLORS; i++) {
        delete palette[i];
    }
}


//
// When one of the tool buttons is pushed, the wTool member is set
// to the button's ID.
//
LRESULT
MiniPaintWin::idcTool(UINT, WPARAM wParam, LPARAM lParam)
{
    if (HIWORD(lParam) == BN_CLICKED) {
        if (wTool == IDC_TEXT)
            wmKillFocus(0, 0, 0L);
        wTool = wParam;
    }
    SetFocus(hwnd());
    return 0L;
}

//
// When one of the color buttons is pushed, the wColor member is set
// to the button's ID.
//
LRESULT
MiniPaintWin::idcColor(UINT, WPARAM wParam, LPARAM lParam)
{
    if (HIWORD(lParam) == BN_CLICKED)
        wColor = wParam;
    SetFocus(hwnd());
    return 0L;
}


//
// When the main window is created, the tool buttons are given their
// bitmaps, the color buttons their colors, and all windows are created.
//
LRESULT
MiniPaintWin::wmCreate(UINT, WPARAM, LPARAM)
{
    DevContext dc(hwnd());
    aFont.getStock(SYSTEM_FONT);
    aFont.select(dc);
    TextMetric tm(dc);
    cxChar = tm.tmAveCharWidth;
    cyChar = tm.tmHeight;

    int i;

    static LPSTR toolNames[MAXTOOLS] = {
        "Pen", "Eraser", "Circle", "Text", "Rect", "Round", 
    };
    for (i=0; i < MAXTOOLS; i++) {
        tools[i] = new OwnerDrawButton
            (new Bitmap(getWord(GWW_HINSTANCE), toolNames[i]));
        tools[i]->create(NULL, getWord(GWW_HINSTANCE),
            WS_VISIBLE, FIRSTTOOL + i, hwnd());
    }

    for (i=0; i < MAXCOLORS; i++) {
        palette[i] = new ColorButton(colors[i]);
        palette[i]->create(NULL, getWord(GWW_HINSTANCE),
            WS_VISIBLE, FIRSTCOLOR + i, hwnd());
    }
    return 0L;
}

//
// When the main window is resized, the size of the tools is adjusted.
//
LRESULT
MiniPaintWin::wmSize(UINT, WPARAM, LPARAM lParam)
{
    int dx = LOWORD(lParam) / 20;
    int dy = HIWORD(lParam) * 2 / (MAXTOOLS + MAXCOLORS + 6);
    int x, y, i;

    for (i=0, x=dx, y=dy; i < MAXTOOLS; i++) {
        tools[i]->move(x, y, dx+1, dy+1, TRUE);
        if (i & 1) {
            x = dx;
            y += dy;
        } else
            x += dx;
    }

    for (i=0, x=dx, y+=dy; i < MAXCOLORS; i++) {
        palette[i]->move(x, y, dx+1, dy+1, TRUE);
        if (i & 1) {
            x = dx;
            y += dy;
        } else
            x += dx;
    }

    // Set the area for drawing
    drawRect.set(dx * 4, dy, dx * 19, HIWORD(lParam) - dy);
    return 0L;
}

//
// Draw the owner-draw button indicated by the control ID.
//
LRESULT
MiniPaintWin::wmDrawItem(UINT, WPARAM, LPARAM lParam)
{
    LPDRAWITEMSTRUCT lpDIS = (LPDRAWITEMSTRUCT) lParam;
    OwnerDrawButton *pODB = (OwnerDrawButton *) findWinHnd(lpDIS->hwndItem);
    pODB->drawItem(lpDIS);
    return 0L;
}

//
// When the user presses the mouse within the drawing area,
// the starting and previous mouse positions are set.
//
LRESULT
MiniPaintWin::wmLButtonDown(UINT, WPARAM, LPARAM lParam)
{
    // Make sure the mouse is within the drawing area
    Point pt(lParam);
    if (!pt.inRect(&drawRect))
        return 0L;

    if (wTool == IDC_TEXT) {
        xCaret = pt.x;
        yCaret = pt.y;
        if (xCaret + cxChar >= drawRect.right)
            xCaret -= cxChar;
        if (yCaret + cyChar >= drawRect.bottom)
            yCaret -= cyChar;
        return wmSetFocus(0, 0, 0L);
    }

    // Remember the current position as the starting and previous point
    ptStart = ptLast = pt;

    // Keep the cursor within the drawing area while draggin
    Rect aRect = drawRect;
    aRect.clientToScreen(hwnd());
    ClipCursor(&aRect);
    return 0L;
}

//
// When the mouse is dragged within the drawing area, the selected
// tool determines the action taken.
//
LRESULT
MiniPaintWin::wmMouseMove(UINT, WPARAM, LPARAM lParam)
{
    if (!ptStart.x)         // Mouse just being moved, not dragged
        return 0L;

    Point pt(lParam);
    DevContext dc(hwnd());
    Region clip(&drawRect);
    HRGN hrgnOld = clip.select(dc);

    switch (wTool) {
        case IDC_PEN:           // Draw from previous spot to current
        case IDC_ERASER:
        {
            Pen aPen;
            if (wTool == IDC_PEN)
                aPen.create(PS_SOLID, 1, colors[wColor - FIRSTCOLOR]);
            else
                aPen.create(PS_SOLID, 10, GetSysColor(COLOR_WINDOW));
            HPEN hpOld = aPen.select(dc);
            MoveTo(dc, ptLast.x, ptLast.y);
            LineTo(dc, pt.x, pt.y);
            SelectObject(dc, hpOld);
            break;
        }

        case IDC_RECT:          // Draw an outline of the shape indicated
        case IDC_ROUND:
        case IDC_CIRCLE:
        {
            Brush aBrush;
            aBrush.getStock(NULL_BRUSH);
            HBRUSH hbrOld = aBrush.select(dc);
            int oldROP2 = SetROP2(dc, R2_NOT);
#if (0 MSC(+1))         
            // Microsoft C++  (requires explict type conversion)
            drawShape(dc, ptStart.toPOINT(), ptLast.toPOINT()); 
            drawShape(dc, ptStart.toPOINT(), pt.toPOINT());     
#elif (0 BCC(+1))       
            // Borland C++
            drawShape(dc, ptStart,          ptLast);
            drawShape(dc, ptStart,          pt);
#endif
            SetROP2(dc, oldROP2);
            SelectObject(dc, hbrOld);
            break;
        }
    }
    ptLast = pt;

    SelectObject(dc, hrgnOld);
    return 0L;
}

//
//
LRESULT
MiniPaintWin::wmLButtonUp(UINT, WPARAM, LPARAM)
{
    if (!ptStart.x)         // Mouse was just moved, not dragged
        return 0L;

    switch (wTool) {
        case IDC_RECT:          // Erase the last outline,
        case IDC_ROUND:         // then paint the shape with color
        case IDC_CIRCLE:
        {
            DevContext dc(hwnd());
            Brush aBrush;
            aBrush.getStock(NULL_BRUSH);
            HBRUSH hbrOld = aBrush.select(dc);
            int oldROP2 = SetROP2(dc, R2_NOT);
#if (0 MSC(+1))
            // Microsoft C++  (requires explict type conversion)
            drawShape(dc, ptStart.toPOINT(), ptLast.toPOINT());
#elif (0 BCC(+1))
            // Borland C++
            drawShape(dc, ptStart, ptLast);
#endif
            SetROP2(dc, oldROP2);

            aBrush.create(colors[wColor - FIRSTCOLOR]);
            aBrush.select(dc);
#if (0 MSC(+1))
            // Microsoft C++  (requires explict type conversion)
            drawShape(dc, ptStart.toPOINT(), ptLast.toPOINT());
#elif (0 BCC(+1))
            // Borland C++
            drawShape(dc, ptStart, ptLast);
#endif

            SelectObject(dc, hbrOld);
            break;
        }
    }

    // Flag that dragging is done and stop restricting the cursor
    ptStart.x = ptStart.y = 0;
    ClipCursor(NULL);
    return 0L;
}

//
// Called when our main window gets the focus (wMsg == WM_SETFOCUS),
// and also when the user clicks in the draw area with the text tool
// (wMsg == 0).
//
LRESULT
MiniPaintWin::wmSetFocus(UINT, WPARAM, LPARAM)
{
    if (wTool == IDC_TEXT) {
        if (!aCaret) {
            aCaret.create(hwnd(), NULL, 2, cyChar);
            aCaret.setPos(xCaret, yCaret);
            aCaret.show();
        } else {
            aCaret.setPos(xCaret, yCaret);
        }
    }
    return 0L;
}

//
// Called when our main window loses the focus (wMsg == WM_KILLFOCUS),
// and also when the user selects a new tool replacing text (wMsg == 0).
//
LRESULT
MiniPaintWin::wmKillFocus(UINT, WPARAM, LPARAM)
{
    if (aCaret) {
        aCaret.hide();
        aCaret.destroy();
    }
    return 0L;
}

//
// If the text tool is selected, display any characters typed, wrapping
// around when we get to the edge of the display.
//
LRESULT
MiniPaintWin::wmChar(UINT, WPARAM wParam, LPARAM lParam)
{
    if (wTool != IDC_TEXT)
        return 0L;

    DevContext dc(hwnd());
    DWORD dwColor = SetTextColor(dc, colors[wColor - FIRSTCOLOR]);
    int iBkMode = SetBkMode(dc, TRANSPARENT);
    aCaret.hide();
    aFont.select(dc);

    for (int i=0; i < LOWORD(lParam); i++) {
        char c = LOBYTE(wParam);  // character to be printed
        int len = GetTextExtent(dc, (LPSTR) &c, 1);
        TextOut(dc, xCaret, yCaret, (LPSTR) &c, 1);
        if ((xCaret += len) + cxChar >= drawRect.right) {
            xCaret = drawRect.left + cxChar;
            if ((yCaret += cyChar) + cyChar >= drawRect.bottom)
                yCaret = drawRect.top + cyChar;
        }
    }

    aCaret.setPos(xCaret, yCaret);
    aCaret.show();
    SetBkMode(dc, iBkMode);
    SetTextColor(dc, dwColor);
    return 0L;
}

//
// Paints a new rectangle over the drawing area, erasing it.
//
LRESULT
MiniPaintWin::wmPaint(UINT, WPARAM, LPARAM)
{
    Paint p(hwnd());

    Brush aBrush;
    aBrush.getStock(NULL_BRUSH);
    HBRUSH hbrOld = aBrush.select(p);

    drawRect.inflate(1, 1);
    Rectangle(p, drawRect.left, drawRect.top, 
        drawRect.right, drawRect.bottom);
    drawRect.inflate(-1, -1);

    SelectObject(p, hbrOld);

    return 0L;
}


//
// This method draws the current shape with the currently selected brush.
// 
void
MiniPaintWin::drawShape(HDC hdc, POINT ptFrom, POINT ptTo)
{
    // Be sure not to give Windows points in reverse
    int x0 = min(ptFrom.x, ptTo.x);
    int x1 = max(ptFrom.x, ptTo.x);
    int y0 = min(ptFrom.y, ptTo.y);
    int y1 = max(ptFrom.y, ptTo.y);

    switch (wTool) {
        case IDC_CIRCLE:
            Ellipse(hdc, x0, y0, x1, y1);
            break;
        case IDC_RECT:
            Rectangle(hdc, x0, y0, x1, y1);
            break;
        case IDC_ROUND:
            int x2 = (x1 - x0) / 4;
            int y2 = (y1 - y0) / 4;
            RoundRect(hdc, x0, y0, x1, y1, x2, y2);
            break;
    }
}


//////////////////////
// WINMAIN FUNCTION //
//////////////////////

//
// The WinMain entry procedure simply creates the derived application object,
// passing its parameters to the constructor, and the runs the application.
//
int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR, int)
{
    MiniPaintWin aWindow;
//  aWindow.setMenuName(aWindow.getName());
    aWindow.setCaption("Mini-Paint Demonstration Program");

    App anApp(hInstance, hPrevInstance);
    return anApp.run(aWindow, aWindow.getName());
}

