//
// POPPAD.CPP   Popup Editor program.
//
// Created:     Bruce Clift  8/23/92
//

#include <windows.h>

#include "window.hpp"
#include "handler.hpp"
#include "edit.hpp"
#include "file.hpp"
#include "filedlg.hpp"
#include "regwincl.hpp"
#include "modal.hpp"
#include "message.hpp"
#include "app.hpp"

#include "poppad.h"


/////////////////////////////////
// MAIN WINDOW CLASS INTERFACE //
/////////////////////////////////

class PopPadMain : public Window {
private:
    static Handler  aHandler;       // Handler for main window messages
    static HACCEL   hAccel;         // Handle to accelerator table

    char *      szUntitled;
    char *      szFileSpec;

    LPSTR       lpszCmdLine;
    Edit        anEdit;
    File        aFile;
    char        szRealFileName[16];
    BOOL        bNeedSave;

public:
    // Default constructor
    PopPadMain();

    // Data member access
    Handler *getHandler(void)   { return &aHandler; }
    char    *getName(void)      { return "PopPad"; }
    HACCEL  getAccel(void)      { return hAccel; }
    void    setAccel(HACCEL ha) { hAccel = ha; }
    void    setCmdLine(LPSTR lpszCL) { lpszCmdLine = lpszCL; }

    // Initialization methods
    void setClassReg(RegWinCls &);
    BOOL display(HINSTANCE hInst, char *szApp, char *, int nCmdShow);

    // General methods
    void okMessageBox(char *szString, char *szFileName);
    BOOL readFile (char *szFileName, BOOL bAskName);
    BOOL writeFile(char *szFileName, BOOL bAskName);
    BOOL printFile(char *szFileName);
    void doCaption(void);
    short askAboutSave(void);

    // Callback methods
    LRESULT wmCreate(UINT, WPARAM, LPARAM);
    LRESULT wmSetFocus(UINT, WPARAM, LPARAM);
    LRESULT wmSize(UINT, WPARAM, LPARAM);
    LRESULT wmInitMenuPopup(UINT, WPARAM, LPARAM);
    LRESULT wmClose(UINT, WPARAM, LPARAM);
    LRESULT wmQueryEndSession(UINT, WPARAM, LPARAM);

    LRESULT idcEdit(UINT, WPARAM, LPARAM);      // Message from edit control
    LRESULT idmNew(UINT, WPARAM, LPARAM);       // File menu operations
    LRESULT idmOpen(UINT, WPARAM, LPARAM);
    LRESULT idmSave(UINT, WPARAM, LPARAM);
    LRESULT idmSaveAs(UINT, WPARAM, LPARAM);
    LRESULT idmPrint(UINT, WPARAM, LPARAM);
    LRESULT idmExit(UINT, WPARAM, LPARAM);
    LRESULT idmUndo(UINT, WPARAM, LPARAM);      // Edit menu operations
    LRESULT idmCut(UINT, WPARAM, LPARAM);
    LRESULT idmCopy(UINT, WPARAM, LPARAM);
    LRESULT idmPaste(UINT, WPARAM, LPARAM);
    LRESULT idmClear(UINT, WPARAM, LPARAM);
    LRESULT idmSelAll(UINT, WPARAM, LPARAM);
    LRESULT idmAbout(UINT, WPARAM, LPARAM);

};

HACCEL  PopPadMain::hAccel = NULL;


//////////////////////////////////////
// INITIALIZATION AND OTHER METHODS //
//////////////////////////////////////

//
// Default constructor initializes data members.
//
PopPadMain::PopPadMain()
    : szUntitled("(untitled)"), szFileSpec("*.TXT"), bNeedSave(FALSE)
{
    szRealFileName[0] = '\0';
}

//
// Adjust the window class registration structure.
//
void
PopPadMain::setClassReg(RegWinCls &rwc)
{
    rwc.hIcon = LoadIcon(rwc.hInstance, getName());
}

BOOL
PopPadMain::display(HINSTANCE hInst, char *szApp, char *, int nCmdShow)
{
    // Pass the command line as a creation parameter
    if (!create(szApp, NULL, hInst, WS_OVERLAPPEDWINDOW, NULL, NULL,
        GetSystemMetrics(SM_CXSCREEN) / 4,
        GetSystemMetrics(SM_CYSCREEN) / 4,
        GetSystemMetrics(SM_CXSCREEN) / 2,
        GetSystemMetrics(SM_CYSCREEN) / 2,
        lpszCmdLine))
            return FALSE;

    show(nCmdShow);
    update();

    setAccel(LoadAccelerators(hInst, szApp));
    return TRUE;
}

//
// Write out a message box which includes a file name.
//
void
PopPadMain::okMessageBox(char *szString, char *szFileName)
{
    char szBuffer[40];
    wsprintf(szBuffer, szString, (LPSTR) szFileName);
    MessageBox(hwnd(), szBuffer, getName(), MB_OK | MB_ICONEXCLAMATION);
}

//
// Read the contents of a file into the edit control.
//
BOOL
PopPadMain::readFile(char *szFileName, BOOL bAskName)
{
    if (bAskName) {
        FileOpenDialog aFOD(&aFile, szFileSpec+1, szFileSpec, 0);
        if (!aFOD.run("FileOpen", hwnd()))
            return FALSE;
        lstrcpy(szFileName, aFOD.getFileName());
    }

    if (!aFile.reopen(OF_READ)) {
        aFile.close();
        okMessageBox("Cannot open file %s", szFileName);
        return FALSE;
    }
    DWORD dwLength = aFile.length();
    if (dwLength >= 32000) {
        aFile.close();
        okMessageBox("File %s too large", szFileName);
        return FALSE;
    }
    HGLOBAL hTextBuffer = GlobalAlloc(GHND, dwLength + 1);
    if (hTextBuffer == NULL) {
        aFile.close();
        okMessageBox("Cannot allocate memory for %s", szFileName);
        return FALSE;
    }

    LPSTR lpTextBuffer = GlobalLock(hTextBuffer);
    aFile.read(lpTextBuffer, (WORD) dwLength);
    aFile.close();
    lpTextBuffer [(WORD) dwLength] = '\0';

    anEdit.setText(lpTextBuffer);
    GlobalUnlock(hTextBuffer);
    GlobalFree(hTextBuffer);
    return TRUE;
}

//
// Write the contents of the edit control into the given file.
//
BOOL
PopPadMain::writeFile(char *szFileName, BOOL bAskName)
{
    if (bAskName) {
        FileSaveDialog aFSD(&aFile, szFileSpec+1, szFileSpec);
        if (!aFSD.run("FileSave", hwnd()))
            return FALSE;
        lstrcpy(szFileName, aFSD.getFileName());
        if (aFSD.getStatus() == 1) {
            char szBuffer[40];
            wsprintf(szBuffer, "Replace existing %s", (LPSTR) szFileName);
            if (IDNO == MessageBox(hwnd(), szBuffer, getName(),
                MB_YESNO | MB_ICONQUESTION))
                    return FALSE;
        }
        else
            aFile.open(szFileName, OF_PARSE);

    }

    if (!aFile.reopen(OF_CREATE)) {
        aFile.close();
        okMessageBox("Cannot create file %s", szFileName);
        return FALSE;
    }
    WORD wLength = anEdit.getTextLength();
    HLOCAL hTextBuffer = anEdit.getHandle();
    NPSTR npTextBuffer = LocalLock(hTextBuffer);

    if (wLength != aFile.write(npTextBuffer, wLength)) {
        aFile.close();
        okMessageBox("Cannot write file %s to disk", szFileName);
        return FALSE;
    }

    aFile.close();
    LocalUnlock(hTextBuffer);
    return TRUE;
}

//
// Print the given file.
//
BOOL
PopPadMain::printFile(char *szFileName)
{
    MessageBox(hwnd(), "Printing not yet implemented", getName(),
        MB_OK | MB_ICONEXCLAMATION);
    return FALSE;
}

//
// Write the application name and the current file name (if any)
// into the window caption.
//
void
PopPadMain::doCaption(void)
{
    char szCaption[40];
    wsprintf(szCaption, "%s - %s", (LPSTR) getName(),
        (LPSTR) (szRealFileName[0] ? szRealFileName : szUntitled));
    setText(szCaption);
}

//
// Ask the user if changes should be saved.
//
short
PopPadMain::askAboutSave(void)
{
    char szBuffer[40];
    wsprintf(szBuffer, "Save current changes: %s",
        (LPSTR) (szRealFileName[0] ? szRealFileName : szUntitled));

    short nReturn;
    if (IDYES == (nReturn = MessageBox(hwnd(), szBuffer, getName(),
        MB_YESNOCANCEL | MB_ICONQUESTION)))
            if (!sendMessage(WM_COMMAND, IDM_SAVE, 0L))
                return IDCANCEL;
    return nReturn;
}


/////////////////////////////////
// CALLBACK TABLES AND METHODS //
/////////////////////////////////

static CB MsgCbList[] = {
    { WM_CREATE,            (CBPROC) &PopPadMain::wmCreate },
    { WM_SETFOCUS,          (CBPROC) &PopPadMain::wmSetFocus },
    { WM_SIZE,              (CBPROC) &PopPadMain::wmSize },
    { WM_INITMENUPOPUP,     (CBPROC) &PopPadMain::wmInitMenuPopup },
    { WM_CLOSE,             (CBPROC) &PopPadMain::wmClose },
    { WM_QUERYENDSESSION,   (CBPROC) &PopPadMain::wmQueryEndSession },
    { 0, NULL }
};

static CB CmdCbList[] = {
    { IDC_EDIT,             (CBPROC) &PopPadMain::idcEdit },
    { IDM_NEW,              (CBPROC) &PopPadMain::idmNew },
    { IDM_OPEN,             (CBPROC) &PopPadMain::idmOpen },
    { IDM_SAVE,             (CBPROC) &PopPadMain::idmSave },
    { IDM_SAVEAS,           (CBPROC) &PopPadMain::idmSaveAs },
    { IDM_PRINT,            (CBPROC) &PopPadMain::idmPrint },
    { IDM_EXIT,             (CBPROC) &PopPadMain::idmExit },
    { IDM_UNDO,             (CBPROC) &PopPadMain::idmUndo },
    { IDM_CUT,              (CBPROC) &PopPadMain::idmCut },
    { IDM_COPY,             (CBPROC) &PopPadMain::idmCopy },
    { IDM_PASTE,            (CBPROC) &PopPadMain::idmPaste },
    { IDM_CLEAR,            (CBPROC) &PopPadMain::idmClear },
    { IDM_SELALL,           (CBPROC) &PopPadMain::idmSelAll },
    { IDM_ABOUT,            (CBPROC) &PopPadMain::idmAbout },
    { 0, NULL }
};

Handler PopPadMain::aHandler(MsgCbList, CmdCbList);

//
// Load the accelerator table (first window only), create
// the multi-line edit control, and read a file whose name may
// have been passed on the command line.
//
LRESULT
PopPadMain::wmCreate(UINT, WPARAM, LPARAM lParam)
{
    LPCREATESTRUCT lpCS = (LPCREATESTRUCT) lParam;

    // Create the main edit control which will fill the window
    anEdit.create(NULL, lpCS->hInstance,
        WS_CHILD | WS_VISIBLE | WS_HSCROLL | WS_VSCROLL | WS_BORDER |
        ES_LEFT | ES_MULTILINE | ES_AUTOHSCROLL | ES_AUTOVSCROLL,
        IDC_EDIT, hwnd());
    anEdit.limitText(32000);

    // Attempt to read a file name passed on the command line
#if (WINVER < 0x030a)
    if (lpCS->lpCreateParams && lstrlen(lpCS->lpCreateParams)) {
        aFile.open(lpCS->lpCreateParams, OF_PARSE);
#else
    if (lpCS->lpCreateParams && lstrlen(LPCSTR(lpCS->lpCreateParams))) {
        aFile.open(LPCSTR(lpCS->lpCreateParams), OF_PARSE);
#endif
        char szFileName[16];
        lstrcpy(szFileName, (LPSTR) AnsiNext(lstrrchr((LPSTR) aFile.szPathName, '\\')));
        if (readFile(szFileName, FALSE))
            lstrcpy(szRealFileName, szFileName);
    }
    doCaption();
    return 0L;
}

//
// Sets the focus to the edit control whenever the main window gets focus.
//
LRESULT
PopPadMain::wmSetFocus(UINT, WPARAM, LPARAM)
{
    SetFocus(anEdit);
    return 0L;
}

//
// Size the edit control when the main window is sized.
//
LRESULT
PopPadMain::wmSize(UINT, WPARAM, LPARAM lParam)
{
    anEdit.move(0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
    return 0L;
}

//
// Enable the edit menu items depending on the state of the edit control
// and the clipboard.
//
LRESULT
PopPadMain::wmInitMenuPopup(UINT, WPARAM wParam, LPARAM lParam)
{
    if (lParam == 1) {
        EnableMenuItem(wParam, IDM_UNDO,
            anEdit.canUndo() ? MF_ENABLED : MF_GRAYED);
        EnableMenuItem(wParam, IDM_PASTE,
            IsClipboardFormatAvailable(CF_TEXT) ? MF_ENABLED : MF_GRAYED);

        int nFirst, nLast;
        anEdit.getSel(nFirst, nLast);
        WORD wEnable = (nFirst == nLast ? MF_GRAYED : MF_ENABLED);
        EnableMenuItem(wParam, IDM_CUT,   wEnable);
        EnableMenuItem(wParam, IDM_COPY,  wEnable);
        EnableMenuItem(wParam, IDM_CLEAR, wEnable);
    }
    return 0L;
}

//
// Ask user before closing window.
//
LRESULT
PopPadMain::wmClose(UINT, WPARAM, LPARAM)
{
    if (!bNeedSave || IDCANCEL != askAboutSave())
        destroy();
    return 0L;
}

//
// Ask user before ending Windows session.
//
LRESULT
PopPadMain::wmQueryEndSession(UINT, WPARAM, LPARAM)
{
    if (!bNeedSave || IDCANCEL != askAboutSave())
        return 1L;
    return 0L;
}


//
// idcEdit processes messages from the edit control.
//
LRESULT
PopPadMain::idcEdit(UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    switch (HIWORD(lParam)) {
    case EN_UPDATE:
        bNeedSave = TRUE;
        return 0L;

    case EN_ERRSPACE:
        MessageBeep(0);
        MessageBox(hwnd(), "Edit control out of space.",
            getName(), MB_OK | MB_ICONSTOP);
        return 0L;
    }
    return callDefProc(wMsg, wParam, lParam);
}

//
// Standard file menu commands
//
LRESULT PopPadMain::idmNew(UINT, WPARAM, LPARAM)
{
    if (bNeedSave && IDCANCEL == askAboutSave())
        return 0L;

    anEdit.setText("\0");
    szRealFileName[0] = '\0';
    doCaption();
    bNeedSave = FALSE;
    return 0L;
}

LRESULT PopPadMain::idmOpen(UINT, WPARAM, LPARAM)
{
    if (bNeedSave && IDCANCEL == askAboutSave())
        return 0L;

    char szFileName[16];
    if (readFile(szFileName, TRUE)) {
        lstrcpy(szRealFileName, szFileName);
        doCaption();
        bNeedSave = FALSE;
    }
    return 0L;
}

LRESULT PopPadMain::idmSave(UINT, WPARAM, LPARAM)
{
    if (!szRealFileName[0])
        return idmSaveAs(0, 0, 0L);

    if (writeFile(szRealFileName, FALSE)) {
        bNeedSave = FALSE;
        return 1L;
    }
    return 0L;
}

LRESULT PopPadMain::idmSaveAs(UINT, WPARAM, LPARAM)
{
    char szFileName[16];
    if (writeFile(szFileName, TRUE)) {
        lstrcpy(szRealFileName, szFileName);
        doCaption();
        bNeedSave = FALSE;
        return 1L;
    }
    return 0L;
}

LRESULT
PopPadMain::idmPrint(UINT, WPARAM, LPARAM)
{
    printFile(szRealFileName[0] ? szRealFileName : szUntitled);
    return 0L;
}

LRESULT
PopPadMain::idmExit(UINT, WPARAM, LPARAM)
{
    return wmClose(WM_CLOSE, 0, 0L);
}

//
// Standard edit menu commands
//
LRESULT
PopPadMain::idmUndo(UINT, WPARAM, LPARAM)
{
    return anEdit.sendMessage(WM_UNDO, 0, 0L);
}

LRESULT
PopPadMain::idmCut(UINT, WPARAM, LPARAM)
{
    return anEdit.sendMessage(WM_CUT, 0, 0L);
}

LRESULT
PopPadMain::idmCopy(UINT, WPARAM, LPARAM)
{
    return anEdit.sendMessage(WM_COPY, 0, 0L);
}

LRESULT
PopPadMain::idmPaste(UINT, WPARAM, LPARAM)
{
    return anEdit.sendMessage(WM_PASTE, 0, 0L);
}

LRESULT
PopPadMain::idmClear(UINT, WPARAM, LPARAM)
{
    return anEdit.sendMessage(WM_CLEAR, 0, 0L);
}

LRESULT
PopPadMain::idmSelAll(UINT, WPARAM, LPARAM)
{
    anEdit.setSel(0, 32767);
    return 0L;
}

//
// idmAbout runs the About Dialog Box.
//
LRESULT
PopPadMain::idmAbout(UINT, WPARAM, LPARAM)
{
    // Construct and run a dialog box object
    Modal about;
    about.run("AboutBox", hwnd());
    return 0L;
}


///////////////////////////////////
// DERIVED CLASS FOR APPLICATION //
///////////////////////////////////

class PopPadApp : public App {
public:
    // Default constructor
    PopPadApp(HINSTANCE hi, HINSTANCE hpi, LPSTR lpszCL, int nCS)
        : App(hi, hpi, lpszCL, nCS) {}

    // Redefine the message loop for an accelerator table
    WORD messageLoop(void);

};

//
// Override the default message loop to pass a pointer to the main window.
// The Message class understands this to mean that an accelerator table
// is to be obtained from the window.
//
WORD
PopPadApp::messageLoop(void)
{
    Message msg;
    return msg.loop(pMain);
}

//
// The WinMain entry procedure simply creates the derived application object,
// passing its parameters to the constructor, and the runs the application.
//
int PASCAL
WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
    PopPadMain aWindow;
    aWindow.setMenuName(aWindow.getName());
    aWindow.setCmdLine(lpszCmdLine);

    PopPadApp anApp(hInstance, hPrevInstance, lpszCmdLine, nCmdShow);
    return anApp.run(aWindow, aWindow.getName());
}
