//
// BITMAP.HPP   Interface for Bitmap, derived from BITMAP struct.
//
// Created:     Bruce Clift  8/17/91
//

#ifndef __BITMAP_HPP
#define __BITMAP_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Bitmap : public BITMAP {
private:
    HBITMAP _hbitmap;           // Bitmap handle

public:
    // Default constructor clears the handle and BITMAP structure
    Bitmap();
    // The no-parameter create expects the user to fill the BITMAP fields
    HBITMAP create(void)
        { return (_hbitmap = CreateBitmapIndirect(this)); }

    // Load constructors and methods load a bitmap resource
    Bitmap(HINSTANCE hInstance, int nRes)       { load(hInstance, nRes); }
    Bitmap(HINSTANCE hInstance, LPCSTR lpRes)   { load(hInstance, lpRes); }
    HBITMAP load(HINSTANCE hInstance, int nRes)
        { return doLoad(LoadBitmap(hInstance, MAKEINTRESOURCE(nRes))); }
    HBITMAP load(HINSTANCE hInstance, LPCSTR lpRes)
        { return doLoad(LoadBitmap(hInstance, lpRes)); }

    // The create constructors and methods manually create a bitmap
    Bitmap(int nWidth, int nHeight, UINT nPlanes,
            UINT nBitsPixel, LPSTR lpBits)
        { create(nWidth, nHeight, nPlanes, nBitsPixel, lpBits); }
    Bitmap(HDC hdc, int nWidth, int nHeight)
        { create(hdc, nWidth, nHeight); }
    HBITMAP create(int nWidth, int nHeight, UINT nPlanes,
            UINT nBitsPixel, LPSTR lpBits);
    HBITMAP create(HDC hdc, int nWidth, int nHeight)
        { return doLoad(CreateCompatibleBitmap(hdc, nWidth, nHeight)); }
    HBITMAP createCompatible(HDC hdc, int nWidth, int nHeight)
        { return doLoad(CreateCompatibleBitmap(hdc, nWidth, nHeight)); }
    HBITMAP createDiscardable(HDC hdc, int nWidth, int nHeight)
        { return doLoad(CreateDiscardableBitmap(hdc, nWidth, nHeight)); }

    // Helper function for above load and create methods
protected:
    HBITMAP doLoad(HBITMAP hb);
public:

    // The destructor and destroy method destroy bitmaps that were created
    ~Bitmap() { destroy(); }
    BOOL destroy(void);

    // Data member access
    HBITMAP hbitmap(void) { return _hbitmap; }
    HBITMAP hbitmap(HBITMAP hb) { return (_hbitmap = hb); }

    // Operator methods
    BOOL operator! (void) { return (_hbitmap == NULL); }
    operator HBITMAP(void) { return (_hbitmap); }

    // Windows convenience methods
    void    createCaret(HWND hw, int nWidth, int nHeight)
        { CreateCaret(hw, _hbitmap, nWidth, nHeight); }
    LONG    setBits(DWORD cBits, LPSTR lpvBits)
        { return SetBitmapBits(_hbitmap, cBits, lpvBits); }
    LONG    getBits(LONG cbBuffer, LPSTR lpvBits)
        { return GetBitmapBits(_hbitmap, cbBuffer, lpvBits); }
    DWORD   setDimension(int x, int y)
        { return SetBitmapDimension(_hbitmap, x, y); }
    DWORD   getDimension(void)          { return GetBitmapDimension(_hbitmap); }
    HBRUSH  createPatternBrush(void)    { return CreatePatternBrush(_hbitmap); }
    HBITMAP select(HDC hdc)             { return SelectObject(hdc, _hbitmap); }
    int     get(void)
        { return GetObject(_hbitmap, sizeof(BITMAP),
            (LPSTR)(BITMAP *) this); }

};  // Bitmap

#endif // __BITMAP_HPP
