//
// BRUSH.HPP    Interface for Brush, derived from LOGBRUSH struct.
//
// Created:     Bruce Clift  8/19/91
//

#ifndef __BRUSH_HPP
#define __BRUSH_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Brush : public LOGBRUSH {
private:
    HBRUSH  _hbrush;        // Brush handle
    BOOL    bStock;         // Is brush a stock object?

public:
    // Default constructor clears the handle and LOGBRUSH structure
    Brush();
    // The no-parameter create expects the user to fill the LOGBRUSH fields
    HBRUSH create(void)
        { bStock=FALSE; return (_hbrush=CreateBrushIndirect(this)); }

    // Other create constructors and methods manually create a brush
    Brush(COLORREF crColor)
        : _hbrush(NULL) { create(crColor); }
    Brush(int nIndex, COLORREF crColor)
        : _hbrush(NULL) { create(nIndex, crColor); }
    Brush(HBITMAP hBitmap)
        : _hbrush(NULL) { create(hBitmap); }
    Brush(HGLOBAL hPackedDIB, WORD wUsage)
        : _hbrush(NULL) { create(hPackedDIB, wUsage); }
    HBRUSH create(COLORREF crColor);
    HBRUSH create(int nIndex, COLORREF crColor);
    HBRUSH create(HBITMAP hBitmap);
    HBRUSH create(HGLOBAL hPackedDIB, WORD wUsage);

    // Get stock method gets a stock brush
    HBRUSH getStock(int nIndex);

    // The destructor and destroy method destroy brushes that were created
    ~Brush() { destroy(); }
    BOOL destroy(void);

    // Data member access
    HBRUSH hbrush(void) { return _hbrush; }
    HBRUSH hbrush(HBRUSH hbr) { return (_hbrush = hbr); }

    // Operator methods
    BOOL operator! (void) { return (_hbrush == NULL); }
    operator HBRUSH(void) { return (_hbrush); }

    // Windows convenience methods
    int     fillRect(HDC hdc, LPCRECT lpR)
                { return FillRect(hdc, lpR, _hbrush); }
    BOOL    fillRgn(HDC hdc, HRGN hrgn)
                { return FillRgn(hdc, hrgn, _hbrush); }
    int     frameRect(HDC hdc, LPCRECT lpR)
                { return FrameRect(hdc, lpR, _hbrush); }
    BOOL    frameRgn(HDC hdc, HRGN hrgn, int nWidth, int nHeight)
                { return FrameRgn(hdc, hrgn, _hbrush, nWidth, nHeight); }
    BOOL    unrealize(void) { return UnrealizeObject(_hbrush); }
    HBRUSH  select(HDC hdc) { return SelectObject(hdc, _hbrush); }
    int     get(void)
        { return GetObject(_hbrush, sizeof(LOGBRUSH),
            (LPSTR)(LOGBRUSH*)this); }

};  // Brush

#endif // __BRUSH_HPP
