//
// CARET.HPP    Interface for Caret class.
//
// Created:     Bruce Clift  9/24/91
//

#ifndef __CARET_HPP
#define __CARET_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Caret {
private:
    HWND    _hw;            // Handle of owner window

public:
    // Default constructor clears instance variable
    Caret() : _hw(NULL) {}

    // The create constructor and method create a caret for a window
    Caret(HWND hw, HBITMAP hBitmap=NULL, int nWidth=0, int nHeight=0)
        { create(hw, hBitmap, nWidth, nHeight); }
    void create(HWND hw, HBITMAP hBitmap=NULL, int nWidth=0, int nHeight=0)
        { CreateCaret(_hw=hw, hBitmap, nWidth, nHeight); }

    // The destructor and destroy method destroy the owner window's caret
    ~Caret(void) { destroy(); }
    void destroy(void) { DestroyCaret(); _hw=NULL; }

    // Data member access
    HWND    hwnd(void)      { return (_hw); }
    HWND    hwnd(HWND hw)   { return (_hw = hw); }

    // Operator methods
    BOOL operator! (void) { return (_hw == NULL); }
    operator HWND(void) { return (_hw); }

    // Windows convenience methods
    UINT getBlinkTime(void)             { return GetCaretBlinkTime(); }
    void setBlinkTime(UINT wMSeconds)   { SetCaretBlinkTime(wMSeconds); }
    void hide(void)                     { HideCaret(_hw); }
    void show(void)                     { ShowCaret(_hw); }
    void setPos(int x, int y)           { SetCaretPos(x, y); }
    void getPos(LPPOINT lpPoint)        { GetCaretPos(lpPoint); }

};  // Caret

#endif // __CARET_HPP
