//
// CONTROL.HPP  Interface for control virtual class.
//
// Created:     Bruce Clift  9/15/91
//

#ifndef __CONTROL_HPP
#define __CONTROL_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

#ifndef __WINDOW_HPP
#include "window.hpp"
#endif

class Handler;


// For controls, default position and size to 0
#undef DEF_X
#undef DEF_Y
#undef DEF_CX
#undef DEF_CY
#define DEF_X   0
#define DEF_Y   0
#define DEF_CX  0
#define DEF_CY  0


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Control : public Window {
protected:
    HWND    _hDlg;          // Handle of parent dialog box
    int     _wID;           // ID of control

public:
    // Default constructor clears instance variables
    Control() : _hDlg(NULL), _wID(NULL), Window() {}

    // Constructor and method to get control in dialog box
    Control(HWND hd, int id) { get(hd, id); }
    HWND get(HWND hd, int id)
        { return hwnd(GetDlgItem(_hDlg=hd, _wID=id)); }

#pragma warn -hid
    // Creation method
    HWND create(LPCSTR name, HINSTANCE hInst, long lStyle,
        WORD id, HWND hwParent,
        int     xx          = DEF_X,
        int     yy          = DEF_Y,
        int     cxx         = DEF_CX,
        int     cyy         = DEF_CY,
        LPSTR   lpParam     = DEF_LPPARAM,
        long    lExStyle    = DEF_EXSTYLE
    ) { return Window::create(name, hInst, addStyle(lStyle),
            _wID=id, _hDlg=hwParent, xx, yy, cxx, cyy, lpParam, lExStyle); }
#pragma warn .hid
    virtual long addStyle(long lStyle) { return (WS_CHILD | lStyle); }

    // Member access functions
    HWND hDlg(HWND hd)  { return (_hDlg = hd); }
    HWND hDlg(void)     { return (_hDlg); }
    int wID(int id)     { return (_wID = id); }
    int wID(void)       { return (_wID); }

    // DONT create a default Handler object like the Window class
    Handler *getHandler(void) { return NULL; }

    // No default message handling for controls
    void regDefMsgs(Handler *) {}

};

#endif // __CONTROL_HPP
