//
// CURSOR.HPP   Interface for Cursor.
//
// Created:     Bruce Clift  8/17/91
//

#ifndef __CURSOR_HPP
#define __CURSOR_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Cursor {
private:
    HCURSOR _hcursor;       // Handle of cursor
    BOOL    bCreated;       // Was cursor created or just loaded?

public:
    // Default constructor clears instance variables
    Cursor();

    // Load constructors and methods load a cursor resource
    Cursor(int nRes)                        { load(nRes); }
    Cursor(LPCSTR lpRes)                    { load(lpRes); }
    Cursor(HINSTANCE hInst, LPCSTR lpName)  { load(hInst, lpName); }
    HCURSOR load(int nRes)      { return load(NULL, MAKEINTRESOURCE(nRes)); }
    HCURSOR load(LPCSTR lpRes)  { return load(NULL, lpRes); }
    HCURSOR load(HINSTANCE hInst, LPCSTR lpName);

    // The create constructor and method manually create a cursor
    Cursor(HINSTANCE hInst, int x, int y, int nWidth, int nHeight,
            LPCSTR lpANDbits, LPCSTR lpXORbits)
        { create(hInst, x, y, nWidth, nHeight, lpANDbits, lpXORbits); }
    HCURSOR create(HINSTANCE hInst, int x, int y, int nWidth, int nHeight,
            LPCSTR lpANDbits, LPCSTR lpXORbits);

    // The destructor and destroy method destroy cursors that were created
    ~Cursor(void) { destroy(); }
    BOOL destroy(void);

    // Data member access
    HCURSOR hcursor(void)       { return (_hcursor); }
    HCURSOR hcursor(HCURSOR hc) { return (_hcursor = hc); }

    // Operator methods
    BOOL operator! (void) { return (_hcursor == NULL); }
    operator HCURSOR (void) { return (_hcursor); }

    // Windows convenience methods
    HCURSOR set(void)               { return SetCursor(_hcursor); }
    int     show(BOOL bShow)        { return ShowCursor(bShow); }
    void    clip(LPRECT lpR)        { ClipCursor(lpR); }
    void    getPos(LPPOINT lpP)     { GetCursorPos(lpP); }
    void    setPos(int x, int y)    { SetCursorPos(x, y); }

};  // Cursor

#endif // __CURSOR_HPP
