//
// DEVCONTE.HPP Interface for DevContext class.
//
// Created:     Bruce Clift  8/23/91
//

#ifndef __DEVCONTE_HPP
#define __DEVCONTE_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class DevContext {
private:
    HDC     _hdc;       // Device context handle
    HWND    _hw;        // Window handle used in get method (NULL if created)

public:
    // Default constructor clears instance variables
    DevContext() : _hdc(NULL), _hw(NULL) {}

    // Get constructor and methods get a window's device context
    DevContext(HWND hw)     { get(hw); }
    HDC get(HWND hw)        { return _hdc=GetDC(_hw=hw); }
    HDC getWindow(HWND hw)  { return _hdc=GetWindowDC(_hw=hw); }

    // Create constructor and methods create a device context
    DevContext(LPCSTR lpDriver, LPCSTR lpDev, LPCSTR lpOutput, LPCSTR lpInit)
        { create(lpDriver, lpDev, lpOutput, lpInit); }
    HDC create(LPCSTR lpDriver, LPCSTR lpDev, LPCSTR lpOutput, LPCSTR lpInit)
        { _hw=NULL; return (_hdc=CreateDC(lpDriver, lpDev, lpOutput, lpInit)); }
    HDC createDisplayDC(void)
        { _hw=NULL; return (_hdc=CreateDC("DISPLAY", NULL, NULL, NULL)); }
    HDC createDisplayIC(void)
        { _hw=NULL; return (_hdc=CreateIC("DISPLAY", NULL, NULL, NULL)); }
    HDC createIC(LPCSTR lpDriver, LPCSTR lpDev, LPCSTR lpOutput, LPCSTR lpInit)
        { _hw=NULL; return (_hdc=CreateIC(lpDriver, lpDev, lpOutput, lpInit)); }
    HDC createCompatible(HDC hdc)
        { _hw=NULL; return (_hdc=CreateCompatibleDC(hdc)); }

    // Creating printer contexts
    HDC createPrinter(void)     { return doPrinter(FALSE); }
    HDC createPrinterIC(void)   { return doPrinter(TRUE); }
private:
    HDC doPrinter(BOOL bInfo);
public:

    // Destructor and destroy method release/delete device context
    ~DevContext() { destroy(); }
    BOOL destroy(void) {
        if (_hdc == NULL) return FALSE;
        if (_hw)    ReleaseDC(_hw, _hdc);   // Release pre-existing contexts
        else    DeleteDC(_hdc);         // Delete created contexts
        _hdc = _hw = NULL;
        return TRUE;
    }

    // Data member access
    HDC     hdc(void) { return _hdc; }
    HDC     hdc(HDC h) { return (_hdc = h); }
    HWND    hwnd(void) { return _hw; }
    HWND    hwnd(HDC h) { return (_hw = h); }

    // Operator methods
    BOOL operator! (void) { return (_hdc == NULL); }
    operator HDC (void) { return (_hdc); }

    // Windows convenience functions
    int     save(void)                  { return SaveDC(_hdc); }
    BOOL    restore(int nSavedDC)       { return RestoreDC(_hdc, nSavedDC); }
    int     getDeviceCaps(int nIndex)   { return GetDeviceCaps(_hdc, nIndex); }
    DWORD   getOrg(void)                { return GetDCOrg(_hdc); }
    BOOL    scroll(int dx, int dy, LPCRECT lprcScroll, LPCRECT lprcClip,
                HRGN hrgnUpdate, LPRECT lprcUpdate)
        { return ScrollDC(_hdc, dx, dy, lprcScroll, lprcClip,
            hrgnUpdate, lprcUpdate); }

};

#endif // __DEVCONTE_HPP
