//
// DIALOG.HPP   Abstract class, base for Modal and Modeless.
//
// Created:     Bruce Clift  7/31/91
//

#ifndef __DIALOG_HPP
#define __DIALOG_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

#ifndef __LIST_HPP
#include "list.hpp"
#endif

#ifndef __WINDOW_HPP
#include "window.hpp"
#endif

class Handler;


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Dialog : public Window {
private:
    static List newCtlList;     // List of new dialog child controls

public:
    // Default constructor just calls Window constructor
    Dialog();

    // Override virtual method in Window: create default Handler object
    // with dptDialog parameter, specifying no default window procedure.
    Handler *getHandler(void);

    // Override virtual method: no default window procedure for dialog boxes.
    LRESULT callDefProc(FARPROC, UINT wMsg, WPARAM, LPARAM);

    // Don't register default callback messages for Window (WM_DESTROY)
    void regDefMsgs(Handler *);


    ///////////////////////////////////////////////////////////////////
    // SUPPORT FOR CHILD WINDOWS RECEIVING MESSAGES (SEE DLGCTL.CPP) //
    ///////////////////////////////////////////////////////////////////

    // Virtual methods to create and destroy list of child controls
    virtual void createChildren(void);
    virtual void destroyChildren(void);

    // New dialog box child control list management
protected:
    void    createNewCtlList(void)  { newCtlList.destroy(); }
    void    destroyNewCtlList(void) { newCtlList.destroy(); }
    BOOL    addNewCtl(WORD wID, Window *pCtl)
        { return newCtlList.insert(wID, (DWORD) (WORD) pCtl); }
public:
    static int      newCtls(void);
    static Window   *newControl(HWND hw);

    // Windows convenience functions
    void setDef(WORD wID) { SendMessage(hwnd(), DM_SETDEFID, wID, 0L); }
    WORD getDef(void) { return SendMessage(hwnd(), DM_GETDEFID, 0, 0L); }

};

#endif // __DIALOG_HPP
