//
// EDIT.HPP     Interface for Edit, derived from Window.
//
// Created:     Bruce Clift  8/26/91
//

#ifndef __EDIT_HPP
#define __EDIT_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

#ifndef __CONTROL_HPP
#include "control.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Edit : public Control {
public:
    // Constructors and destructors
    Edit() : Control() {}
    Edit(HWND hd, int id) : Control(hd, id) {}

    // Creation method alterations
    LPCSTR getClassName(void) { return "edit"; }

    // Message functions for single- or multiple-line edits
    BOOL    canUndo(void)
        { return SendMessage(hwnd(), EM_CANUNDO, 0, 0L); }
    void    emptyUndoBuffer(void)
        { SendMessage(hwnd(), EM_EMPTYUNDOBUFFER, 0, 0L); }
    BOOL    getModify(void)
        { return SendMessage(hwnd(), EM_GETMODIFY, 0, 0L); }
    HLOCAL  getHandle(void)
        { return SendMessage(hwnd(), EM_GETHANDLE, 0, 0L); }
    void    getRect(LPRECT lpR)
        { SendMessage(hwnd(), EM_GETRECT, 0, (LONG) lpR); }
    void    getSel(int &nFirst, int &nLast)
        { long l=SendMessage(hwnd(), EM_GETSEL, 0, 0L);
            nFirst=LOWORD(l); nLast=HIWORD(l); }
    void    limitText(int nMax)
        { SendMessage(hwnd(), EM_LIMITTEXT, nMax, 0L); }
    int     lineLength(int nIndex)
        { return SendMessage(hwnd(), EM_LINELENGTH, nIndex, 0L); }
    void    replaceSel(LPCSTR lpsz)
        { SendMessage(hwnd(), EM_REPLACESEL, 0, (LONG) lpsz); }
    void    setModify(BOOL bModify)
        { SendMessage(hwnd(), EM_SETMODIFY, bModify, 0L); }
    void    setPasswordChar(WORD wChar)
        { SendMessage(hwnd(), EM_SETPASSWORDCHAR, wChar, 0L); }
    void    setSel(int nFirst, int nLast)
        { SendMessage(hwnd(), EM_SETSEL, 0, MAKELONG(nFirst, nLast)); }
    BOOL    undo(void)
        { return SendMessage(hwnd(), EM_UNDO, 0, 0L); }

    // Message functions for multiple-line edit controls
    BOOL    fmtLines(BOOL bWordWrap)
        { return SendMessage(hwnd(), EM_FMTLINES, bWordWrap, 0L); }
    int     getLine(int nLine, WORD wMax, LPSTR lpsz)
        {   *((WORD *) lpsz) = wMax; // Put max length in first word of buffer
            return SendMessage(hwnd(), EM_GETLINE, nLine, (LONG) lpsz); }
    int     getLineCount(void)
        { return SendMessage(hwnd(), EM_GETLINECOUNT, 0, 0L); }
    int     lineFromChar(int nIndex)
        { return SendMessage(hwnd(), EM_LINEFROMCHAR, nIndex, 0L); }
    int     lineIndex(int nLine)
        { return SendMessage(hwnd(), EM_LINEINDEX, nLine, 0L); }
    void    lineScroll(int cLines, int cChars)
        { SendMessage(hwnd(), EM_LINESCROLL, 0, MAKELONG(cLines, cChars)); }
    void    setHandle(HLOCAL hBuffer)
        { SendMessage(hwnd(), EM_SETHANDLE, hBuffer, 0L); }
    void    setRect(LPRECT lpR)
        { SendMessage(hwnd(), EM_SETRECT, 0, (LONG) lpR); }
    void    setRectNP(LPRECT lpR)
        { SendMessage(hwnd(), EM_SETRECTNP, 0, (LONG) lpR); }
    BOOL    setTabStops(int cTabs, int FAR *a)
        { return SendMessage(hwnd(), EM_SETTABSTOPS, cTabs, (LONG) a); }
    void    setWordBreak(FARPROC func)
        { SendMessage(hwnd(), EM_SETWORDBREAK, 0, (LONG) func); }

};

#endif // __EDIT_HPP
