//
// FONT.HPP     Interface for Font, derived from LOGFONT struct.
//
// Created:     Bruce Clift  8/17/91
//

#ifndef __FONT_HPP
#define __FONT_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Font : public LOGFONT {
private:
    HFONT   _hfont;         // Font handle
    BOOL    bStock;         // Is font a stock object?

public:
    // Default constructor clears instance variables and LOGFONT struct
    Font();
    // The no-parameter create expects the caller to fill LOGFONT fields
    HFONT create(void);

    // Get stock constructor and method get a stock font
    Font(int nIndex) { getStock(nIndex); }
    HFONT getStock(int nIndex);

    // The create constructor and method manually create a font
    Font(int nHt, int nWid, int nEscape, int nOrient, int nWt,
        BYTE cItal, BYTE cUnder, BYTE cStrike, BYTE cChSet, BYTE cOutPrec,
        BYTE cClipPrec, BYTE cQual, BYTE cPitchFam, LPCSTR lpFace)
            { create(nHt, nWid, nEscape, nOrient, nWt,
                cItal, cUnder, cStrike, cChSet, cOutPrec,
                cClipPrec, cQual, cPitchFam, lpFace); }
    HFONT create(int nHt, int nWid, int nEscape, int nOrient, int nWt,
        BYTE cItal, BYTE cUnder, BYTE cStrike, BYTE cChSet, BYTE cOutPrec,
        BYTE cClipPrec, BYTE cQual, BYTE cPitchFam, LPCSTR lpFace);

    // The destructor and destroy method destroy fonts that were created
    ~Font() { destroy(); }
    BOOL destroy(void);

    // Data member access
    HFONT hfont(void) { return _hfont; }
    HFONT hfont(HFONT hf) { return (_hfont = hf); }

    // Operator methods
    BOOL operator! (void) { return (_hfont == NULL); }
    operator HFONT (void) { return (_hfont); }

    // Windows convenience methods
    HFONT   select(HDC hdc) { return SelectObject(hdc, _hfont); }
    int     get(void)
        { return GetObject(_hfont, sizeof(LOGFONT),
            (LPSTR)(LOGFONT *) this); }

};  // Font


//////////////////////////////////
// INLINE METHOD IMPLEMENTATION //
//////////////////////////////////

//
// The creation method sets the LOGFONT structure fields to the parameters
// passed, then calls the no-parameter create method.
//
inline HFONT
Font::create(int nHt, int nWid, int nEscape, int nOrient, int nWt,
    BYTE cItal, BYTE cUnder, BYTE cStrike, BYTE cChSet, BYTE cOutPrec,
    BYTE cClipPrec, BYTE cQual, BYTE cPitchFam, LPCSTR lpFace)
{
    lfHeight            = nHt;
    lfWidth             = nWid;
    lfEscapement        = nEscape;
    lfOrientation       = nOrient;
    lfWeight            = nWt;
    lfItalic            = cItal;
    lfUnderline         = cUnder;
    lfStrikeOut         = cStrike;
    lfCharSet           = cChSet;
    lfOutPrecision      = cOutPrec;
    lfClipPrecision     = cClipPrec;
    lfQuality           = cQual;
    lfPitchAndFamily    = cPitchFam;
    lstrcpy((LPSTR) lfFaceName, lpFace);
    return create();
}

#endif // __FONT_HPP
