//
// HANDLER.HPP  Interface for Handler class.
//
// Created:     Bruce Clift  7/21/91
//

#ifndef __HANDLER_HPP
#define __HANDLER_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

#ifndef __WINDOW_HPP
#include "window.hpp"
#endif

#ifndef __CBPROC_HPP
#include "cbproc.hpp"
#endif

#ifndef __LIST_HPP
#include "list.hpp"
#endif


///////////
// TYPES //
///////////

// Structure to hold message and associated callback method.
// This struct is only used for an application's callback method tables.
typedef struct tagCB {
    WORD    wIndex;
    CBPROC  cb;
} CB;
typedef CB      *PCB;
typedef CB NEAR *NPCB;
typedef CB FAR  *LPCB;

// Enum type to determine the type of default window procedure.
typedef enum { dptWindow, dptDialog, dptMDIFrame, dptMDIChild } DEFPROCTYPE;

// Window procedures (different for standard and registered classes)
extern "C" LRESULT FAR PASCAL EXPORT winProcStd(HWND, UINT, WPARAM, LPARAM);
extern "C" LRESULT FAR PASCAL EXPORT winProcReg(HWND, UINT, WPARAM, LPARAM);


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Handler {
private:
    static FARPROC  lpfnWPStd;      // Procedure instance of winProcStd
    static Handler  *pNewHandler;   // Pointer to new handler for winProcReg
protected:
    List        *pMsgs, *pCmds;     // Message and command callback lists
    FARPROC     defWinProc;         // Default window procedure

public:
    // Default constructor builds callback lists
    Handler(PCB pMsgList=NULL, PCB pCmdList=NULL, DEFPROCTYPE dpt=dptWindow);

    // Destructor frees list memory
    ~Handler();

    // Data member access
    FARPROC getDefWinProc(void) { return defWinProc; }
    FARPROC getWinProcStd(HINSTANCE hInst);
    CBPROC *getMsgCB(WORD wMsg);
    CBPROC *getCmdCB(WORD wCmd);

    // Routine to store Handler object pointer for winProcReg
    void setNewHand(LPCSTR lpClass, HINSTANCE hInst);

    // Un/register callback functions for messages and WM_COMMAND identifiers
    BOOL regMsgCb(PCB pMsgList, BOOL bOverRide=TRUE);
    BOOL regCmdCb(PCB pCmdList, BOOL bOverRide=TRUE);
    BOOL regMsgCb(WORD wMsg, CBPROC cb, BOOL bOverRide=TRUE);
    BOOL regCmdCb(WORD wId,  CBPROC cb, BOOL bOverRide=TRUE);
    BOOL unRegMsgCb(WORD wMsg);
    BOOL unRegCmdCb(WORD wId);

private:
    // Methods to set and get info from registered class's class extra bytes
    void            setPointer(HWND hwnd);
    static Handler *getPointer(HWND hwnd);

    // Virtual methods to set/get other extra info (useful for superclassing)
protected:
    virtual void setClsExtraBytes(HWND);
    virtual void getClsExtraBytes(HWND);

    // Window procedures for Windows to call
    friend LRESULT FAR PASCAL EXPORT winProcStd(HWND, UINT, WPARAM, LPARAM);
    friend LRESULT FAR PASCAL EXPORT winProcReg(HWND, UINT, WPARAM, LPARAM);

}; // Handler

#endif // __HANDLER_HPP
