//
// ICON.HPP     Interface for Icon class.
//
// Created:     Bruce Clift  8/17/91
//

#ifndef __ICON_HPP
#define __ICON_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Icon {
private:
    HICON   _hicon;         // Handle of icon
    BOOL    bCreated;       // Was icon created or just loaded?

public:
    // Default constructor clears instance variables
    Icon();

    // Load constructors and methods load a icon resource
    Icon(int nRes)                          { load(nRes); }
    Icon(LPCSTR lpRes)                      { load(lpRes); }
    Icon(HINSTANCE hInst, LPCSTR lpName)    { load(hInst, lpName); }
    HICON load(int nRes)        { return load(NULL, MAKEINTRESOURCE(nRes)); }
    HICON load(LPCSTR lpRes)    { return load(NULL, lpRes); }
    HICON load(HINSTANCE hInst, LPCSTR lpName);

    // The create constructor and method manually create a icon
    Icon(HINSTANCE hInst, int nWidth, int nHeight,
        BYTE nPlanes, BYTE nBitsPixel, LPCSTR lpANDbits, LPCSTR lpXORbits)
            { create(hInst, nWidth, nHeight,
                nPlanes, nBitsPixel, lpANDbits, lpXORbits); }
    HICON create(HINSTANCE hInst, int nWidth, int nHeight,
            BYTE nPlanes, BYTE nBitsPixel, LPCSTR lpANDbits, LPCSTR lpXORbits) {
        bCreated = TRUE;
        return (_hicon = CreateIcon(hInst, nWidth, nHeight,
            nPlanes, nBitsPixel, lpANDbits, lpXORbits));
    }

    // The destructor and destroy method destroy icons that were created
    ~Icon(void) { destroy(); }
    BOOL destroy(void);

    // Data member access
    HICON hicon(void)       { return (_hicon); }
    HICON hicon(HICON hi)   { return (_hicon = hi); }

    // Operator methods
    BOOL operator! (void) { return (_hicon == NULL); }
    operator HICON (void) { return (_hicon); }

    // Windows convenience methods
    BOOL draw(HDC hdc, int x, int y)
        { return DrawIcon(hdc, x, y, _hicon); }

};  // Icon

#endif // __ICON_HPP
