//
// LISTBOX.HPP  Interface for ListBox, derived from Window.
//
// Created:     Bruce Clift  8/26/91
//

#ifndef __LISTBOX_HPP
#define __LISTBOX_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

#ifndef __CONTROL_HPP
#include "control.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class ListBox : public Control {
public:
    // Constructors and destructors
    ListBox() : Control() {}
    ListBox(HWND hd, int id) : Control(hd, id) {}

    // Creation method alterations
    LPCSTR getClassName(void) { return "listbox"; }
    
    // Message functions
    int     addString(LPCSTR lpsz)
        { return SendMessage(hwnd(), LB_ADDSTRING, 0, (LONG) lpsz); }
    int     deleteString(int nIndex)
        { return SendMessage(hwnd(), LB_DELETESTRING, nIndex, 0L); }
    int     dir(WORD wAttribute, LPCSTR lpFileSpec)
        { return 1+SendMessage(hwnd(), LB_DIR, wAttribute, (LONG) lpFileSpec); }
    int     findString(int nIndex, LPCSTR lpPrefix)
        { return SendMessage(hwnd(), LB_FINDSTRING, nIndex, (LONG) lpPrefix); }
    int     getCount(void)
        { return SendMessage(hwnd(), LB_GETCOUNT, 0, 0L); }
    int     getHorizontalExtent(void)
        { return SendMessage(hwnd(), LB_GETHORIZONTALEXTENT, 0, 0L); }
    LONG    getItemData(int nIndex)
        { return SendMessage(hwnd(), LB_GETITEMDATA, nIndex, 0L); }
    int     getItemRect(int nIndex, LPRECT lpR)
        { return SendMessage(hwnd(), LB_GETITEMRECT, nIndex, (LONG) lpR); }
    int     getSel(int nIndex)
        { return SendMessage(hwnd(), LB_GETSEL, nIndex, 0L); }
    int     getText(int nIndex, LPSTR lpsz)
        { return SendMessage(hwnd(), LB_GETTEXT, nIndex, (LONG) lpsz); }
    int     getTextLen(int nIndex)
        { return SendMessage(hwnd(), LB_GETTEXTLEN, nIndex, 0L); }
    int     getTopIndex(void)
        { return SendMessage(hwnd(), LB_GETTOPINDEX, 0, 0L); }
    int     insertString(int nIndex, LPCSTR lpsz)
        { return SendMessage(hwnd(), LB_INSERTSTRING, nIndex, (LONG) lpsz); }
    int     resetContent(void)
        { return SendMessage(hwnd(), LB_RESETCONTENT, 0, 0L); }
    void    setColumnWidth(int nWidth)
        { SendMessage(hwnd(), LB_SETCOLUMNWIDTH, nWidth, 0L); }
    void    setHorizontalExtent(int nPixels)
        { SendMessage(hwnd(), LB_SETHORIZONTALEXTENT, nPixels, 0L); }
    int     setItemData(int nIndex, LONG lData)
        { return SendMessage(hwnd(), LB_SETITEMDATA, nIndex, lData); }
    BOOL    setTabStops(int cTabs, LPCINT a)
        { return SendMessage(hwnd(), LB_SETTABSTOPS, cTabs, (LONG) a); }
    int     setTopIndex(int nIndex)
        { return SendMessage(hwnd(), LB_SETTOPINDEX, nIndex, 0L); }

    // Single-selection list boxes only
    int     getCurSel(void)
        { return SendMessage(hwnd(), LB_GETCURSEL, 0, 0L); }
    int     setCurSel(int nIndex)
        { return SendMessage(hwnd(), LB_SETCURSEL, nIndex, 0L); }
    int     selectString(int nIndex, LPCSTR lpPrefix)
        { return SendMessage(hwnd(), LB_SELECTSTRING, nIndex, (LONG)lpPrefix); }

    // Multiple-selection list boxes only
    int     getSelCount(void)
        { return SendMessage(hwnd(), LB_GETSELCOUNT, 0, 0L); }
    int     getSelItems(int nMax, int FAR *a)
        { return SendMessage(hwnd(), LB_GETSELITEMS, nMax, (LONG) a); }
    int     selItemRange(BOOL bSelect, WORD wFirst, WORD wLast)
        { return SendMessage(hwnd(), LB_SELITEMRANGE,
            bSelect, MAKELONG(wFirst, wLast)); }
    int     setSel(BOOL bSelect, int nIndex)
        { return SendMessage(hwnd(), LB_SETSEL, bSelect, (LONG) nIndex); }

};

#endif // __LISTBOX_HPP
