//
// MDI.HPP      Class interface for MDI frame and child classes.
//
// Created:     Bruce Clift  10/6/91
//
// Many of the MDI classes' features were inspired by Jeffrey Richter's
// Windows 3: A Developer's Guide.
//

#ifndef __MDI_HPP
#define __MDI_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

#ifndef __WINDOW_HPP
#include "window.hpp"
#endif

class Handler;
class MDIChild;


////////////////////////////////
// FRAME WINDOW CLASS MESSAGE //
////////////////////////////////

#define FM_CHILDDESTROY (WM_USER + 99)


///////////////////////////////
// MDI FRAME CLASS INTERFACE //
///////////////////////////////

class MDIFrame : public Window {
private:
    HWND    _hwClient;      // Handle to MDI Client window

protected:
    HACCEL  _haccel;        // Handle to current accelerator table

public:
    // Default constructor simply clears instance variables
    MDIFrame();

    // Include clip children style for frame windows
    long getStyle(void) { return (WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN); }

    // Create the client window during setChildren
    BOOL setChildren(void);
    virtual HWND createClient(HINSTANCE hInst, HWND hwFrame);

    // Get menu information from derived frame class
    virtual HMENU   getWindowMenu(void) = 0;
    virtual WORD    getFirstChild(void) = 0;
    virtual long    getClientStyle(void);

    // Member access functions
    HWND hwClient(HWND hw)      { return (_hwClient = hw); }
    HWND hwClient(void)         { return (_hwClient); }
    HACCEL setAccel(HACCEL ha)  { return (_haccel = ha); }
    HACCEL getAccel(void)       { return (_haccel); }

    // Register default messages
    void regDefMsgs(Handler *pHand);
    LRESULT wmNCCreate(UINT, WPARAM, LPARAM);
    LRESULT wmSize(UINT, WPARAM, LPARAM);
    LRESULT wmClose(UINT, WPARAM, LPARAM);
    LRESULT wmQueryEndSession(UINT, WPARAM, LPARAM);
    LRESULT wmEndSession(UINT, WPARAM, LPARAM);
    LRESULT wmSysCommand(UINT, WPARAM, LPARAM);
    LRESULT wmNCLButtonDblClk(UINT, WPARAM, LPARAM);
    LRESULT wmCommand(UINT, WPARAM, LPARAM);
    LRESULT fmChildDestroy(UINT, WPARAM, LPARAM);

    // Call default window procedure for MDI frames
    LRESULT callDefProc(UINT wMsg, WPARAM wParam, LPARAM lParam)
        { return DefFrameProc(hwnd(), _hwClient, wMsg, wParam, lParam); }
    LRESULT callDefProc(FARPROC, UINT wMsg, WPARAM wParam, LPARAM lParam)
        { return DefFrameProc(hwnd(), _hwClient, wMsg, wParam, lParam); }

    // Frame utility methods
    Window *        getActiveChild(BOOL *pbMax = NULL);
    virtual void    resizeClient(void);
    virtual void    changeMDIMenu(HMENU hm, WORD wID);
    virtual void    tileVertically(void);

}; // MDIFrame


///////////////////////////////
// MDI CHILD CLASS INTERFACE //
///////////////////////////////

class MDIChild : public Window {
protected:
    MDIFrame *pFrame;   // Pointer back to owner frame window

public:
    // Default constructor simply clears instance variables
    MDIChild();

#pragma warn -hid
    // Creation method lets user specify CreateWindow parameters
    HWND create(LPCSTR cls, LPCSTR title, HINSTANCE hInst, LONG lStyle,
        HWND hwCli,
        int     xx  = CW_USEDEFAULT,
        int     yy  = CW_USEDEFAULT,
        int     cxx = CW_USEDEFAULT,
        int     cyy = CW_USEDEFAULT,
        LONG    lp  = 0L
    );
#pragma warn .hid
    
    // Register default messages
    void regDefMsgs(Handler *pHand);
    LRESULT wmClose(UINT, WPARAM, LPARAM);
    LRESULT wmDestroy(UINT, WPARAM, LPARAM);

    // Status bar callback methods
    void regSBMsgs(Handler *pHand);
    LRESULT wmMenuSelect(UINT, WPARAM, LPARAM);
    LRESULT wmEnterIdle(UINT, WPARAM, LPARAM);

    // Call default window procedure for MDI children
    LRESULT callDefProc(UINT wMsg, WPARAM wParam, LPARAM lParam)
        { return DefMDIChildProc(hwnd(), wMsg, wParam, lParam); }
    LRESULT callDefProc(FARPROC, UINT wMsg, WPARAM wParam, LPARAM lParam)
        { return DefMDIChildProc(hwnd(), wMsg, wParam, lParam); }

}; // MDIChild

#endif // __MDI_HPP
