//
// MENU.HPP     Interface for Menu class.
//
// Created:     Bruce Clift  8/17/91
//

#ifndef __MENU_HPP
#define __MENU_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Menu {
private:
    HMENU   _hmenu;         // Handle of menu
    HWND    _hw;            // Handle of owner window
    BOOL    bCreated;       // Was menu created manually?

public:
    // Default constructor clears instance variables
    Menu();

    // Load constructors and methods load a menu resource
    Menu(LPCSTR lpRes)                      { load(lpRes); }
    Menu(HINSTANCE hInst, LPCSTR lpName)    { load(hInst, lpName); }
    HMENU load(int nRes)        { return load(NULL, MAKEINTRESOURCE(nRes)); }
    HMENU load(LPCSTR lpRes)    { return load(NULL, lpRes); }
    HMENU load(HINSTANCE hInst, LPCSTR lpName);

    // Get constructors and methods get an existing menu
    Menu(HWND hw)                       { get(hw); }
    Menu(HMENU hmParent, int nPos)      { getSub(hmParent, nPos); }
    HMENU get(HWND hw)
        { bCreated=FALSE; return (_hmenu=GetMenu(_hw=hw)); }
    HMENU getSystem(HWND hw, BOOL bRevert)
        { bCreated=FALSE; return (_hmenu=GetSystemMenu(_hw=hw, bRevert)); }
    HMENU getSub(HMENU hmParent, int nPos)
        { bCreated=FALSE; return (_hmenu=GetSubMenu(hmParent, nPos)); _hw=NULL; }

    // The create method manually creates a menu
    HMENU   create(void)        { return doCreate(CreateMenu()); }
protected:
    // Method to do the work for create (and derived class creates)
    HMENU   doCreate(HMENU hm);
public:

    // The destructor and destroy method destroy menus that were created
    ~Menu(void) { destroy(); }
    BOOL destroy(void);

    // Data member access
    HMENU   hmenu(void)     { return (_hmenu); }
    HMENU   hmenu(HMENU hm) { return (_hmenu = hm); }
    HWND    hwnd(void)      { return (_hw); }
    HWND    hwnd(HWND hw)   { return (_hw = hw); }

    // Operator methods
    BOOL operator! (void) { return (_hmenu == NULL); }
    operator HMENU (void) { return (_hmenu); }

    // Dynamic menu building
    BOOL    insert(UINT wPos, UINT wFlags, UINT wID, LPCSTR lpItem)
                { return InsertMenu(_hmenu, wPos, wFlags, wID, lpItem); }
    BOOL    append(UINT wFlags, UINT wID, LPCSTR lpItem)
                { return AppendMenu(_hmenu, wFlags, wID, lpItem); }
    BOOL    modify(UINT wPos, UINT wFlags, UINT wID, LPCSTR lpItem)
                { return ModifyMenu(_hmenu, wPos, wFlags, wID, lpItem); }
    BOOL    remove(UINT wPos, UINT wFlags)
                { return RemoveMenu(_hmenu, wPos, wFlags); }
    BOOL    delMenu(UINT wPos, UINT wFlags)
                { return DeleteMenu(_hmenu, wPos, wFlags); }

    // Menu item methods
    BOOL    checkItem(UINT wID, UINT wCheck)
                { return CheckMenuItem(_hmenu, wID, wCheck); }
    BOOL    enableItem(UINT wID, UINT wEnable)
                { return EnableMenuItem(_hmenu, wID, wEnable); }
    BOOL    hiliteItem(HWND hw, UINT wID, UINT wHilite)
                { return HiliteMenuItem(hw, _hmenu, wID, wHilite); }
    UINT    getItemID(int nPos) { return GetMenuItemID(_hmenu, nPos); }
    int     getItemCount(void)  { return GetMenuItemCount(_hmenu); }
    BOOL    setItemBitmaps(UINT wPos, UINT wFlags,
        HBITMAP hbmpUnchecked, HBITMAP hbmpChecked)
            { return SetMenuItemBitmaps(_hmenu, wPos, wFlags,
                hbmpUnchecked, hbmpChecked); }

    // Other Windows convenience methods
    void    drawBar(void)       { DrawMenuBar(_hw); }
    BOOL    set(HWND hw)        { return SetMenu(hw, _hmenu); }
    int     getString(UINT wID, LPSTR lpS, int nMax, UINT wFlag)
                { return GetMenuString(_hmenu, wID, lpS, nMax, wFlag); }
    UINT    getState(UINT wID, UINT wFlags)
                { return GetMenuState(_hmenu, wID, wFlags); }
    DWORD   getCheckMarkDimensions(void)
                { return GetMenuCheckMarkDimensions(); }

};  // Menu

#endif // __MENU_HPP
