//
// METAFILE.HPP Interface for MetaFile class.
//
// Created:     Bruce Clift  8/23/91
//

#ifndef __METAFILE_HPP
#define __METAFILE_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class MetaFile {
private:
    HDC         _hdc;       // Handle of metafile in creation process
    HMETAFILE   _hmf;       // Ready-to-play Metafile handle

public:
    // Default constructor clears instance variables
    MetaFile();

    // CopyMetaFile constructor and method copy an existing metafile
    MetaFile(HMETAFILE hmf1, LPCSTR lpFile) : _hdc(NULL)
        { copy(hmf1, lpFile); }
    HMETAFILE copy(HMETAFILE hmf1, LPCSTR lpFile)
        { return (_hmf=CopyMetaFile(hmf1, lpFile)); }

    // Other methods used in getting/creating metafiles
    HDC create(LPCSTR lpFile)       { return (_hdc=CreateMetaFile(lpFile)); }
    HMETAFILE close(void)
        { _hmf=CloseMetaFile(_hdc); _hdc=NULL; return (_hmf); }
    HMETAFILE get(LPCSTR lpFile)    { return (_hmf=GetMetaFile(lpFile)); }
    HMETAFILE setBits(HGLOBAL hMem) { return (_hmf=SetMetaFileBits(hMem)); }

    // Destructor and destroy method close or delete the meta file, as needed
    ~MetaFile() { destroy(); }
    BOOL destroy(void);

    // Data member access
    HDC         hdc(void)   { return (_hdc); }
    HDC         hdc(HDC h)  { return (_hdc = h); }
    HMETAFILE   hmf(void)   { return (_hmf); }
    HMETAFILE   hmf(HDC h)  { return (_hmf = h); }

    // Operator methods
    BOOL operator! (void)   { return (!_hdc && !_hmf); }
    operator HANDLE (void)  { return ((HANDLE) (_hdc ? _hdc : _hmf)); }

    // Windows convenience functions
    BOOL    play(HDC h)     { return PlayMetaFile(h, _hmf); }
    HGLOBAL getBits(void)   { return GetMetaFileBits(_hmf); }

};

#endif // __METAFILE_HPP
