//
// PALETTE.HPP  Palette interface, derived from LOGPALETTE struct.
//
// Created:     Bruce Clift  8/19/91
//

#ifndef __PALETTE_HPP
#define __PALETTE_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Palette : public LOGPALETTE {
private:
    HPALETTE    _hpalette;      // Palette handle
    BOOL        bStock;         // Is palette a stock object?

public:
    // Default constructor clears the handle and LOGPALETTE structure
    Palette();

    // The no-parameter create expects the user to fill the LOGPALETTE fields
    HPALETTE create(void)
        { bStock=FALSE; return (_hpalette=CreatePalette(this)); }

    // Get stock method gets the stock default palette
    HPALETTE getStock(void);

    // The destructor and destroy method destroy palettes that were created
    ~Palette() { destroy(); }
    void destroy(void);

    // Data member access
    HPALETTE hpalette(void)         { return (_hpalette); }
    HPALETTE hpalette(HPALETTE hp)  { return (_hpalette = hp); }

    // Operator methods
    BOOL operator! (void) { return (_hpalette == NULL); }
    operator HPALETTE(void) { return (_hpalette); }

    // Windows convenience methods
    int     get(void)
        { return GetObject(_hpalette, sizeof(LOGPALETTE),
                (LPSTR)(LOGPALETTE *) this); }
    HPALETTE select(HDC hdc, BOOL bForceBackground)
        { return SelectPalette(hdc, _hpalette, bForceBackground); }
    void    animate(UINT wStartIndex, UINT wNumEnts, LPPALETTEENTRY lpPE)
        { AnimatePalette(_hpalette, wStartIndex, wNumEnts, lpPE); }
    UINT    setEntries(UINT wStartIndex, UINT wNumEnts, LPPALETTEENTRY lpPE)
        { return SetPaletteEntries(_hpalette, wStartIndex, wNumEnts, lpPE); }
    UINT    getEntries(UINT wStartIndex, UINT wNumEnts, LPPALETTEENTRY lpPE)
        { return GetPaletteEntries(_hpalette, wStartIndex, wNumEnts, lpPE); }
    UINT    getNearestIndex(COLORREF crColor)
        { return GetNearestPaletteIndex(_hpalette, crColor); }
    BOOL    resize(UINT nNumEntries)
        { return ResizePalette(_hpalette, nNumEntries); }

};  // Palette

#endif // __PALETTE_HPP
