//
// POINT.HPP    Interface for Point class, derived from Windows' POINT struct.
//
// Created:     Bruce Clift  8/17/91
//

#ifndef __POINT_HPP
#define __POINT_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Point : public POINT {
public:
    // Constructors fill POINT fields
    Point() {}
    Point(POINT p) { x=p.x; y=p.y; }
    Point(long l) { *((POINT *) this) = MAKEPOINT(l); }
    Point(int xx, int yy) { x = xx; y = yy; }

    // Overloaded operators
    operator    POINT (void)      { return (*((POINT *) this)); } 
    // above cast does not work with MSC, toPOINT does explicit conversion
    POINT       toPOINT(void)     { return (*((POINT *) this)); } 

    // Convenience methods for point API functions
    BOOL    inRect(LPCRECT lpR)
        { return PtInRect(lpR, (*((POINT *) this))); }
    BOOL    inRegion(HRGN hrgn)         { return PtInRegion(hrgn, x, y); }
    void    getBrushOrg(HDC hdc)
        { DWORD dw=GetBrushOrg(hdc); x=LOWORD(dw); y=HIWORD(dw); }
    DWORD   setBrushOrg(HDC hdc)        { return SetBrushOrg(hdc, x, y); }
    void    getCursorPos(void)          { GetCursorPos(this); }
    void    getCaretPos(void)           { GetCaretPos(this); }
    void    setCursorPos(void)          { SetCursorPos(x, y); }
    void    setCaretPos(void)           { SetCaretPos(x, y); }
    void    clientToScreen(HWND hwnd)   { ClientToScreen(hwnd, this); }
    void    screenToClient(HWND hwnd)   { ScreenToClient(hwnd, this); }
    BOOL    visible(HDC hdc)            { return PtVisible(hdc, x, y); }
    HWND    windowFrom(void)
        { return WindowFromPoint(*((POINT *) this)); }
    HWND    childWindowFrom(HWND hwndParent) 
        { return ChildWindowFromPoint(hwndParent, *((POINT *) this)); }

};  // Point

#endif // __POINT_HPP
