//
// PROFILE.HPP  Windows .ini file support.
//
// Created:     Bruce Clift  11/22/91
//

#ifndef __PROFILE_HPP
#define __PROFILE_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

#ifndef __FILE_HPP
#include "file.hpp"
#endif


////////////
// MACROS //
////////////

#define MAXLINELEN 256
#define MAXFILELEN (64L * 1024 - 2)


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Profile : public File {
private:
    static char *szWinIni;          // "win.ini" default file name
    static char *szDefDelim;        // "," default field delimeters

    struct profileFlags {
        unsigned    preLoad : 1;    // Preloading all file data?
        unsigned    ok      : 1;    // Ready to roll?
        unsigned    fresh   : 1;    // Just read in a new line?
        profileFlags() : preLoad(FALSE), ok(FALSE), fresh(TRUE) {}
    } f;

    HGLOBAL hMem;                   // Buffer for entire file
    char    szLine [MAXLINELEN +1]; // Buffer for current line
    WORD    wLinePos;               // Starting position of current line
    WORD    wFilePos;               // Current file position
    WORD    wFileLen;               // File length

public:
    Profile();

    // init method sets file name and preload flag
    BOOL init(char *szPath = szWinIni, BOOL bPreLoad = FALSE);

    // Add data to a profile.
    // If szApp == NULL, the data is added to the current location.
    BOOL setInt    (char *szApp, char *szKey, int nVal);
    BOOL setString (char *szApp, char *szKey, char *szVal);

    // Retrieve data from a profile.
    // To use getField, call getString first with szVal == NULL.
    int  getInt    (char *szApp, char *szKey, int nDef=0);
    int  getString (char *szApp, char *szKey, char *szDef=NULL,
            char *szVal=NULL, int len=0);
    int  getField(char *szVal, int len, char *szDelim=szDefDelim);

    // Status reporting
    BOOL ok(void)           { return ( f.ok); }
    operator BOOL (void)    { return ( f.ok); }
    BOOL operator! (void)   { return (!f.ok); }

private:
    // Utility methods
    BOOL findApp(char *szApp);
    BOOL findKey(char *szKey, BOOL bApp=FALSE);
    char *getValPtr(void);

    void newApp(char *szApp);
    void newLine(BOOL bNew, char *szFmt, ...);

    // File operations
    BOOL open(LPCSTR szPath, UINT wFlags = OF_READWRITE);
    long seek(long lOffset, int iOrigin = 0);
    UINT read(HPVOID lpBuffer, UINT wBytes);
    UINT write(HPCVOID lpBuffer, UINT wBytes);
    BOOL getLine(void);

};

#endif // __PROFILE_HPP
