//
// RECT.HPP     Interface for Rect class, derived from Windows' RECT struct.
//
// Created:     Bruce Clift  8/17/91
//

#ifndef __RECT_HPP
#define __RECT_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Rect : public RECT {
public:
    // Constructors fill RECT fields
    Rect() {}
    Rect(HWND hw) { GetClientRect(hw, this); }
    Rect(RECT &r) { *(RECT *)this = r; }
    Rect(POINT p1, POINT p2)
        { SetRect(this, p1.x, p1.y, p2.x, p2.y); }
    Rect(int x1, int y1, int x2, int y2)
        { SetRect(this, x1, y1, x2, y2); }

    int width (void) { return (right - left); }
    int height(void) { return (bottom - top); }
    void clientToScreen(HWND hw) {
        ClientToScreen(hw, (LPPOINT) &left);
        ClientToScreen(hw, (LPPOINT) &right);
    }

    // Convenience methods for rectangle API functions
    void    copyFrom(LPCRECT lpR)   { CopyRect(this, lpR); }
    void    copyTo(LPRECT lpR)      { CopyRect(lpR, this); }
    BOOL    equal(LPCRECT lpR)      { return EqualRect(this, lpR); }
    void    inflate(int x, int y)   { InflateRect(this, x, y); }
    int     intersect(LPCRECT lpR1, LPCRECT lpR2)
                { return IntersectRect(this, lpR1, lpR2); }
    BOOL    isEmpty(void)           { return IsRectEmpty(this); }
    void    offset(int x, int y)    { OffsetRect(this, x, y); }
    BOOL    ptIn(POINT p)           { return PtInRect(this, p); }
    void    set(int x1, int y1, int x2, int y2)
                { SetRect(this, x1, y1, x2, y2); }
    void    setEmpty(void)          { SetRectEmpty(this); }
    int     unionRect(LPCRECT lpR1, LPCRECT lpR2)
                { return UnionRect(this, lpR1, lpR2); }
    BOOL    visible(HDC hdc)        { return RectVisible(hdc, this); }

    // Other API function convenience methods
    void    invalidate(HWND hw, BOOL bErase)
                { InvalidateRect(hw, this, bErase); }
    void    adjustWindow(DWORD dwStyle, BOOL bMenu)
                { AdjustWindowRect(this, dwStyle, bMenu); }
    void    adjustWindowEx(DWORD dwStyle, BOOL bMenu, DWORD dwExStyle)
                { AdjustWindowRectEx(this, dwStyle, bMenu, dwExStyle); }
    HRGN    createRgnIndirect(void)     { return CreateRectRgnIndirect(this); }
    void    drawFocus(HDC hdc)          { DrawFocusRect(hdc, this); }
    int     excludeClip(HDC hdc)
                { return ExcludeClipRect(hdc, left, top, right, bottom); }
    int     fill(HDC hdc, HBRUSH hbr)   { return FillRect(hdc, this, hbr); }
    int     frame(HDC hdc, HBRUSH hbr)  { return FrameRect(hdc, this, hbr); }
    void    getClient(HWND hw)          { GetClientRect(hw, this); }
    void    getWindow(HWND hw)          { GetWindowRect(hw, this); }
    BOOL    getUpdate(HWND hw, BOOL bErase)
                { return GetUpdateRect(hw, this, bErase); }
    BOOL    inRegion(HRGN hrgn)         { return RectInRegion(hrgn, this); }
    void    invert(HDC hdc)             { InvertRect(hdc, this); }
    void    mapDialog(HWND hw)          { MapDialogRect(hw, this); }
    void    validate(HWND hw)           { ValidateRect(hw, this); }

};  // Rect

#endif // __RECT_HPP
