//
// REGION.HPP   Interface for Region class.
//
// Created:     Bruce Clift  8/19/91
//

#ifndef __REGION_HPP
#define __REGION_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif


/////////////////////
// CLASS INTERFACE //
/////////////////////

class Region {
private:
    HRGN    _hrgn;          // Region handle

public:
    // Default constructor clears the handle
    Region();

    // Create constructors call methods to create a region
    Region(HRGN hSrc1, HRGN hSrc2, int nMode)
        { create(_hrgn, hSrc1, hSrc2, nMode); }
    Region(int x1, int y1, int x2, int y2)
        { create(x1, y1, x2, y2); }
    Region(LPCRECT lpR)
        { create(lpR); }
    Region(LPCPOINT lpPoints, int nCount, int nMode)
        { create(lpPoints, nCount, nMode); }
    Region(LPCPOINT lpPoints, LPINT lpPolyCounts, int nCount, int nMode)
        { create(lpPoints, lpPolyCounts, nCount, nMode); }
    Region(int x1, int y1, int x2, int y2, int x3, int y3)
        { create(x1, y1, x2, y2, x3, y3); }

    // Create methods support above constructors, creating regions
    HRGN create(HRGN hSrc1, HRGN hSrc2, int nMode)
        { return (_hrgn=CombineRgn(_hrgn, hSrc1, hSrc2, nMode)); }
    HRGN create(int x1, int y1, int x2, int y2)
        { return (_hrgn=CreateRectRgn(x1, y1, x2, y2)); }
    HRGN create(LPCRECT lpR)
        { return (_hrgn=CreateRectRgnIndirect(lpR)); }
    HRGN create(LPCPOINT lpPoints, int nCount, int nMode)
        { return (_hrgn=CreatePolygonRgn(lpPoints, nCount, nMode)); }
    HRGN create(LPCPOINT lpPoints, LPCINT lpPolyCounts, int nCount, int nMode)
        { return (_hrgn=CreatePolyPolygonRgn(lpPoints, lpPolyCounts,
                nCount, nMode)); }
    HRGN create(int x1, int y1, int x2, int y2, int x3, int y3)
        { return (_hrgn=CreateRoundRectRgn(x1, y1, x2, y2, x3, y3)); }

    // Other create methods have no corresponding constructor
    HRGN createRect(int x1, int y1, int x2, int y2)
        { return (_hrgn=CreateRectRgn(x1, y1, x2, y2)); }
    HRGN createRect(LPCRECT lpR)
        { return (_hrgn=CreateRectRgnIndirect(lpR)); }
    HRGN createElliptic(int x1, int y1, int x2, int y2)
        { return (_hrgn=CreateEllipticRgn(x1, y1, x2, y2)); }
    HRGN createElliptic(LPCRECT lpR)
        { return (_hrgn=CreateEllipticRgnIndirect(lpR)); }

    // The destructor and destroy method destroy regions that were created
    ~Region() { destroy(); }
    BOOL destroy(void);

    // Data member access
    HRGN hrgn(void) { return _hrgn; }
    HRGN hrgn(HRGN hr) { return (_hrgn = hr); }

    // Operator methods
    BOOL operator! (void) { return (_hrgn == NULL); }
    operator HRGN  (void) { return (_hrgn); }

    // Windows convenience methods
    BOOL    equal(HRGN hr)              { return EqualRgn(_hrgn, hr); }
    BOOL    fill(HDC hdc, HBRUSH hbr)   { return FillRgn(hdc, _hrgn, hbr); }
    BOOL    frame(HDC hdc, HBRUSH hbr, int nWidth, int nHeight)
        { return FrameRgn(hdc, _hrgn, hbr, nWidth, nHeight); }
    int     getBox(LPRECT lpR)          { return GetRgnBox(_hrgn, lpR); }
    int     getUpdate(HWND hw, BOOL fErase)
        { return GetUpdateRgn(hw, _hrgn, fErase); }
    void    invalidate(HWND hw, BOOL bErase)
        { InvalidateRgn(hw, _hrgn, bErase); }
    BOOL    invert(HDC hdc)             { return InvertRgn(hdc, _hrgn); }
    int     offset(int x, int y)        { return OffsetRgn(_hrgn, x, y); }
    BOOL    paint(HDC hdc)              { return PaintRgn(hdc, _hrgn); }
    BOOL    ptIn(int x, int y)          { return PtInRegion(_hrgn, x, y); }
    BOOL    rectIn(LPCRECT lpR)         { return RectInRegion(_hrgn, lpR); }
    HRGN    select(HDC hdc)             { return SelectObject(hdc, _hrgn); }
    int     selectClip(HDC hdc)         { return SelectClipRgn(hdc, _hrgn); }
    void    setRect(int x1, int y1, int x2, int y2)
        { SetRectRgn(_hrgn, x1, y1, x2, y2); }
    void    validate(HWND hw)           { ValidateRgn(hw, _hrgn); }

};  // Region

#endif // __REGION_HPP
