//
// SPRHAND.HPP  Interface for superclassed message handlers.
//
// Created:     Bruce Clift  9/10/91
//

#ifndef __SPRHAND_HPP
#define __SPRHAND_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

#ifndef __HANDLER_HPP
#include "handler.hpp"
#endif

class RegWinCls;


/////////////////////
// CLASS INTERFACE //
/////////////////////

class SuperHandler : public Handler {
private:
    // Indexes of superclass information in class extra bytes
    static const int nBCWinProc, nBCClsExtra, nBCWndExtra;
    // Number of bytes needed by superclass in class extra bytes
    static const int cbSprHandClsExtra;

    // Number of class & window extra bytes in base window class
    int cbBCClsExtra, cbBCWndExtra;

public:
    // Constructors and destructors
    SuperHandler(PCB pMsgList=NULL, PCB pCmdList=NULL);
    ~SuperHandler();

    // The superClass method registers the superclassed window class
    BOOL superClass(RegWinCls *pRWC, int cbClsMore, int cbWndMore);

    // Override virtual methods to set/get class extra bytes info
    void setClsExtraBytes(HWND hwnd);
    void getClsExtraBytes(HWND hwnd);

    // Get index for get/set class/window word/long extra byte functions
    // (Windows values have negative indexes, user value indexes are >=0)
    int indexCls(int n) { return (n<0 ? n : (n + cbBCClsExtra)); }
    int indexWnd(int n) { return (n<0 ? n : (n + cbBCWndExtra)); }

};  // SuperHandler

//
// The following macro overrides Window class methods for the get/set
// class/window word/long functions.  The methods use the number of extra bytes
// in a super class's base class to calculate the index to give Windows API
// functions.  If this macro is used, it MUST be included in the interface of a
// derived Window class, and the parameter MUST be a SuperHandler object.
//
#define DEFINE_EXTRA_BYTE_METHODS(sh) \
    WORD getClassWord(int n) \
        { return GetClassWord(hwnd(), (sh).indexCls(n)); } \
    LONG getClassLong(int n) \
        { return GetClassLong(hwnd(), (sh).indexCls(n)); } \
    WORD setClassWord(int n, WORD w) \
        { return SetClassWord(hwnd(), (sh).indexCls(n), w); } \
    LONG setClassLong(int n, LONG l) \
        { return SetClassLong(hwnd(), (sh).indexCls(n), l); } \
    WORD getWord(int n) \
        { return GetWindowWord(hwnd(), (sh).indexWnd(n)); } \
    LONG getLong(int n) \
        { return GetWindowLong(hwnd(), (sh).indexWnd(n)); } \
    WORD setWord(int n, WORD w) \
        { return SetWindowWord(hwnd(), (sh).indexWnd(n), w); } \
    LONG setLong(int n, LONG l) \
        { return SetWindowLong(hwnd(), (sh).indexWnd(n), l); }

#endif // __SPRHAND_HPP
