//
// STATUSBA.HPP Interface for StatusBar.
//
// Created:     Bruce Clift  11/8/91
//
// Inspired by Jeffrey Richter's Windows 3: A Developer's Guide.
//

#ifndef __STATUSBA_HPP
#define __STATUSBA_HPP

#ifndef __WFDEFS_HPP
#include "wfdefs.hpp"
#endif

class Window;
class Paint;


/////////////////////
// CLASS INTERFACE //
/////////////////////

class StatusBar {
protected:
    Window *    pOwner;         // Window object owning status bar
    Window *    pHelper;        // Window object supplying help
    int         _iHelp;         // Help index

public:
    // Default constructor clears instance variables
    StatusBar();

    // Overloaded operators
    BOOL operator! (void) { return (pOwner == NULL); }
    operator BOOL  (void) { return (pOwner != NULL); }

    // Sets window owner and register status bar messages
    void set(Window *pWin);
    void wmPaint(Paint &p);     // Helper methods
    void wmEnterIdle(void);

    // Save the help index for later use
    void setMenuHelpIndex(Window *pWin, int i);
    int getCurrentHelpIndex(void) { return _iHelp; }

    // Methods to provide status bar help index values
    virtual int getHelpIndex(Window *);
    virtual int getMenuHelpIndex(Window *, int);
    virtual int getSysMenuHelpIndex(Window *, int);
    virtual int getPopupHelpIndex(Window *, int);

    // Convert the saved help index into a help string
    virtual char *getHelp(int);
    virtual char *getHelp(void);

    // Get dimensions of the status bar
    virtual void getRect(RECT &r);

    // Invalidates the status bar area of the screen
    void invalidate(BOOL bErase = TRUE);

    // Writes a string into the status bar
    virtual void textOut(Paint &p, LPSTR lpS);

protected:
    // Draw dividing line
    virtual WORD drawDivide(Paint &p);

};

#endif // __STATUSBA_HPP
