//
// APP.CPP      Class implementation for App.
//
// Created:     Bruce Clift  10/20/91
//

#include <windows.h>
#pragma hdrstop

#include "app.hpp"
#include "window.hpp"


//
// Default constructor sets WinMain parameters and clears pointers.
//
App::App(HINSTANCE hi, HINSTANCE hpi, LPSTR cl, int cs)
    : hInstance(hi), hPrevInstance(hpi), lpCmdLine(cl), nCmdShow(cs),
    pMain(NULL), szName(NULL)
{}


//
// The initialization routines are called by the run method to
// initialize the first instance of an application (initApp),
// later instances (initAnother), and every instance (initInst).
// They may be overridden in derived App classes.
//
BOOL
App::initApp(void)
{
    if (!pMain) return TRUE;
    return (pMain->regClass(hInstance, szName));
}

BOOL
App::initAnother(void)
{
    return TRUE;
}

BOOL
App::initInst(void)
{
    if (!pMain) return TRUE;
    return (pMain->display(hInstance, szName, NULL, nCmdShow));
}


//
// The usual run method to call sets the application's main window,
// and its name.  The main window will be registered and created
// automatically in the App initialization methods.  The name is
// used as the main window's class name.
//
WORD
App::run(Window &pMainWindow, char *szAppName)
{
    pMain = &pMainWindow;
    szName = szAppName;
    return run();
}

//
// run should be called by WinMain immediately after the derived
// App object's constructor.  This routine calls virtual methods
// to initialize the first instance of an application (initApp),
// later instances (initAnother), and all instances (initInst),
// and finally another virtual method to run the message loop.
// Therefore, almost the entire Windows application execution
// happens in this routine. 
//
WORD
App::run(void)
{
    // If no ptr to main window has been set, create default window
    if (!pMain)
        if (!(pMain = new Window()))
            return 0;                   // Must have main window!

    // If no application name yet, use default
    if (!szName)
        szName = "WindowFrame";

    // Different initialization for first and later instances
    if (!(hPrevInstance ? initAnother() : initApp()))
        return 0;

    // Initialization for every instance
    if (!initInst())
        return 0;

    // Process Windows messages
    return messageLoop();
}
