//
// BITMAP.CPP   Methods for Bitmap class.
//
// Created:     Bruce Clift     11/25/91
//              Greg Voss       5/10/93   adapt for MSC 7.0
//

#include <windows.h>
#include <memory.h>

#include "bitmap.hpp"


//
// Default constructor clears the handle and BITMAP structure.
//
Bitmap::Bitmap() : _hbitmap(NULL)
{
    memset((BITMAP *) this, 0, sizeof(BITMAP));
}

//
// This create methods creates a Windows bitmap and fill the fields in the
// BITMAP structure.
//
HBITMAP
Bitmap::create(int nWidth, int nHeight, UINT nPlanes,
    UINT nBitsPixel, LPSTR lpBits)
{
    bmType = 0;
    bmWidthBytes = ((bmWidth * bmBitsPixel >> 3) + 1) &~ 1;
    return (_hbitmap = CreateBitmap(
        bmWidth = nWidth,
        bmHeight = nHeight,
        bmPlanes = nPlanes,
        bmBitsPixel = nBitsPixel,
        bmBits = lpBits
    ));
}

//
// Helper function does the work for load and create methods.
//
HBITMAP
Bitmap::doLoad(HBITMAP hb)
{
    _hbitmap = hb;
    get();
    bmBits = NULL;
    return _hbitmap;
}

//
// The destroy method destroy bitmaps that were created.
//
BOOL
Bitmap::destroy(void)
{
    if (!_hbitmap)
        return FALSE;
    DeleteObject(_hbitmap);
    _hbitmap = NULL;
    return TRUE;
}

