//
// BRUSH.CPP    Methods for Brush class.
//
// Created:     Bruce Clift     11/25/91
//              Greg Voss       5/10/93   adapt for MSC 7.0
//

#include <windows.h>
#include <memory.h>

#include "brush.hpp"


//
// Default constructor clears the handle and LOGBRUSH structure.
//
Brush::Brush() : _hbrush(NULL), bStock(FALSE)
{
    memset((LOGBRUSH *) this, 0, sizeof(LOGBRUSH));
}

//
// Get stock method gets a stock brush.
//
HBRUSH
Brush::getStock(int nIndex)
{
    if (_hbrush)
        destroy();

    bStock = TRUE;
    _hbrush = GetStockObject(nIndex);
    get();
    return _hbrush;
}

//
// The create methods create a brush and fill in the LOGBRUSH structure fields.
//
HBRUSH
Brush::create(COLORREF crColor)
{
    if (_hbrush)
        destroy();

    bStock = FALSE;
    lbHatch = 0;
    lbStyle = BS_SOLID;
    return (_hbrush = CreateSolidBrush(lbColor = crColor));
}

HBRUSH
Brush::create(int nIndex, COLORREF crColor)
{
    if (_hbrush)
        destroy();

    bStock = FALSE;
    lbStyle = BS_HATCHED;
    return (_hbrush = CreateHatchBrush(lbHatch=nIndex, lbColor=crColor));
}

HBRUSH
Brush::create(HBITMAP hBitmap)
{
    if (_hbrush)
        destroy();

    bStock = FALSE;
    lbStyle = BS_PATTERN;
    lbColor = 0L;
    return (_hbrush = CreatePatternBrush(lbHatch = hBitmap));
}

HBRUSH
Brush::create(HGLOBAL hPackedDIB, WORD wUsage)
{
    if (_hbrush)
        destroy();

    bStock = FALSE;
    lbStyle = BS_DIBPATTERN;
    return (_hbrush = CreateDIBPatternBrush(lbHatch=hPackedDIB, lbColor=wUsage));
}

//
// The destroy method destroy brushes that were created.
//
BOOL
Brush::destroy(void)
{
    if (!_hbrush)
        return FALSE;
    if (!bStock)
        DeleteObject(_hbrush);
    else
        bStock = FALSE;
    _hbrush = NULL;
    return TRUE;
}
