//
// DIALOG.CPP   Implementation for Dialog C++ class.
//
// Created:     Bruce Clift  9/20/91
//

#include <windows.h>
#pragma hdrstop

#include "dialog.hpp"
#include "handler.hpp"


//
// Default constructor just calls Window constructor
//
Dialog::Dialog() : Window()
{}

//
// If getHandler is called and the data member _pHand is NULL, getHandler
// creates a Handler object to process messages.  This method should be
// overridden in derived classes!
//
Handler *
Dialog::getHandler(void)
{
    // If handler pointer already set return its value
    if (_pHand)
        return _pHand;

    // Otherwise create default handler for dialog box
    return new Handler(NULL, NULL, dptDialog);
}

//
// callDefProc overrides the Window class virtual method: no default window
// procedure for dialog boxes.
// RETURNS: For WM_INITDIALOG, TRUE means focus has not been set to a child
// control. For other messages, FALSE means the message was not processed.
LRESULT
Dialog::callDefProc(FARPROC, UINT wMsg, WPARAM, LPARAM)
{
    return (wMsg == WM_INITDIALOG);
}

//
// Don't register default callback messages for Window (WM_DESTROY).
//
void
Dialog::regDefMsgs(Handler *)
{}
