//
// DLGCTL.CPP   Support for dialog box controls which want to get messages.
//
// Created:     Bruce Clift  11/25/91
//
// This module is separated since it is always required with Handler
// (even if an app has no dialogs).
//

#include <windows.h>
#pragma hdrstop

#include "list.hpp"
#include "dialog.hpp"
#include "window.hpp"


////////////////////////
// STATIC DATA MEMBER //
////////////////////////

List Dialog::newCtlList;        // List of new dialog child controls


/////////////////////////////////
// VIRTUAL METHODS TO OVERRIDE //
/////////////////////////////////

//
// createChildren should be overridden in derived Dialog classes whose dialog
// boxes contain child controls belonging to window classes registered by the
// application (and thus using the Handler class to process messages).  It
// should first call the createNewCtlList method to initialize the list of new
// controls, then call addNewCtl for each child control using Handler.
//
void
Dialog::createChildren(void)
{
    createNewCtlList();
}

//
// destroyChildren need only be overridden in derived Dialog classes if the
// Control objects added to the list in createChildren were allocated
// dynamically.  If it is overridden the destroyNewCtlList method MUST still be
// called to reset the list of child controls!!!
//
void
Dialog::destroyChildren(void)
{
    destroyNewCtlList();
}


//////////////////////////////////
// STATIC (NON-VIRTUAL) METHODS //
//////////////////////////////////

//
// newControl adds a new dialog child control to the window handle list.
//  INPUT: hw is the window handle.
//  RETURNS: a pointer to a Window object, or NULL for failure.
//
Window *
Dialog::newControl(HWND hw)
{
    // Get this control's ID
    WORD wID = GetWindowWord(hw, GWW_ID);
    if (!wID) return NULL;

    // Find Control object pointer associated with ID
    Window *pWin = (Window *) newCtlList.item(wID);
    if (!pWin) return NULL;

    // Add control to list of created windows
    return pWin->addWinHnd(hw);
}

//
// Returns the number of new controls wanting to receive messages in the
// dialog box which is in the process of being created.
//
int
Dialog::newCtls(void)
{
    return newCtlList;
}
