//
// FILE.CPP     Windows file management support.
//
// Created:     Bruce Clift  11/22/91
//

#include <windows.h>
#pragma hdrstop
#include <io.h>

#include "file.hpp"


//
// Constructors.
//
File::File() : hFile(-1), _wFlags(0)
{}


/////////////////////
// FILE OPERATIONS //
/////////////////////

BOOL
File::open(LPCSTR szPath, UINT wFlags)
{
    // Open the file
    if ((hFile = OpenFile(szPath, this, _wFlags = wFlags)) < 0)
        return FALSE;
    nErrCode = 0;
    return TRUE;
}

BOOL
File::close(void)
{
    if (hFile < 0) return FALSE;
    _lclose(hFile);
    hFile = -1;
    return TRUE;
}

BOOL
File::reopen(void)
{ return (open(NULL, _wFlags | OF_REOPEN)); }

BOOL
File::reopen(UINT wFlags)
{ return (open(NULL, wFlags | OF_REOPEN)); }

long
File::seek(long lOffset, int iOrigin)
{
    if (hFile < 0) return FALSE;
    return (_llseek(hFile, lOffset, iOrigin));
}

//
// Basic I/O methods.  The value of the buffer passed is first tested
// to make sure it is not a NULL pointer.
//
UINT
File::read(HPVOID lpBuffer, UINT wBytes)
{
    if (hFile < 0 || lpBuffer == NULL)
        return -1;
    return (_lread(hFile, lpBuffer, wBytes));
}

UINT
File::write(HPCVOID lpBuffer, UINT wBytes)
{
    if (hFile < 0 || lpBuffer == NULL)
        return -1;
    return (_lwrite(hFile, lpBuffer, wBytes));
}


//
// Report the length of the file, whether open or not.
//
long
File::length(void)
{
    BOOL bOpened = FALSE;
    if (hFile < 0) {
        if (!reopen())
            return -1L;
        bOpened = TRUE;
    }

    long lPos = seek(0L, 1);    // Save position
    long lLen = seek(0L, 2);    // Seek to end to read length
    seek(lPos);                 // Restore position
    
    if (bOpened) close();
    return lLen;
}

    
int  
File::unlink(void)
{
#if (0 MSC(+1))         // Microsoft C++
    return ::_unlink((const char*)szPathName);
#elif (0 BCC(+1))       // Borland C++
    return ::unlink((const char*)szPathName);
#endif
}

