//
// FILEDLG.CPP  Implementation of File menu dialog box class.
//
// Created:     Bruce Clift  8/22/92
//

#include <windows.h>
#pragma hdrstop

#include "wfdefs.hpp"
#if (0 MSC(+1))         // Microsoft C++
#include <direct.h>
#define MAXPATH 80
#elif (0 BCC(+1))       // Borland C++
#include <dir.h>
#endif



#include "filedlg.hpp"
#include "file.hpp"
#include "handler.hpp"
#include "edit.hpp"
#include "pushbutt.hpp"

#include "filedlg.h"


///////////////////////////////////////////////
// METHODS FOR GENERIC FILE DIALOG BOX CLASS //
///////////////////////////////////////////////

//
// Default constructor just calls Dialog constructor.
//
FileDialog::FileDialog(File *pF, char *szExt, char *szSpec, WORD wAttr)
    : pFile(pF), wFileAttr(wAttr)
{
    lstrcpy(szDefExt, szExt);
    lstrcpy(szFileSpec, szSpec);
}

//
// Fill the list boxes in the file dialog box.
//
int
FileDialog::updateLBs(LPSTR szName)
{
    if (DlgDirList(hwnd(), szName, IDC_FLIST, IDC_PATH, wFileAttr))
        if (DlgDirList(hwnd(), szName, IDC_DLIST, 0, 0xc010))
            return TRUE;
    return FALSE;
}

//
// regDefMsgs registers standard File menu dialog box callback methods.
//
void
FileDialog::regDefMsgs(Handler *pHand)
{
    static CB MsgCbList[] = {
        { WM_INITDIALOG,    (CBPROC) &FileDialog::wmInitDialog },
        { 0, NULL }
    };
    static CB CmdCbList[] = {
        { IDC_FNAME,        (CBPROC) &FileDialog::idcFname },
        { IDC_FLIST,        (CBPROC) &FileDialog::idcList },
        { IDC_DLIST,        (CBPROC) &FileDialog::idcList },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regMsgCb(MsgCbList, FALSE);
    pHand->regCmdCb(CmdCbList, FALSE);

    // Register base class' default messages
    Modal::regDefMsgs(pHand);
}

//
// When the file dialog is first opened, the files and directories
// list boxes are filled.
//
LRESULT
FileDialog::wmInitDialog(UINT, WPARAM, LPARAM)
{
    Edit fnEdit(hwnd(), IDC_FNAME);
    fnEdit.limitText(80);
    updateLBs(szFileSpec);
    fnEdit.setText(lstrcpy(szFileName, szFileSpec));
    return TRUE;
}

//
// When the file name is changed, the OK button is enabled only if it is
// not empty.
//
LRESULT
FileDialog::idcFname(UINT, WPARAM wParam, LPARAM lParam)
{
    if (HIWORD(lParam) == EN_CHANGE) {
        PushButton ok(hwnd(), IDOK);
        Edit fnEdit(hwnd(), wParam);
        ok.enable((BOOL) fnEdit.getTextLength());
    }
    return TRUE;
}

//
// This method processes notifications from both the files 
// and directories list boxes.
// When the selection is changed, the file name edit window is updated.
// A double-click is the same as pressing the OK button.
//
LRESULT
FileDialog::idcList(UINT, WPARAM wParam, LPARAM lParam)
{
    switch (HIWORD(lParam)) {
    case LBN_SELCHANGE: {
        if (DlgDirSelect(hwnd(), szFileName, wParam))
            lstrcat(szFileName, szFileSpec);
        Edit fnEdit(hwnd(), IDC_FNAME);
        fnEdit.setText(szFileName);
        fnEdit.setSel(0, 0x7fff);
        return TRUE; }

    case LBN_DBLCLK:
        return sendMessage(WM_COMMAND, IDOK, 0L);
    }
    return FALSE;
}


////////////////////////////////////////////
// METHODS FOR FILE OPEN DIALOG BOX CLASS //
////////////////////////////////////////////

//
// Default constructor just calls Dialog constructor.
//
FileOpenDialog::FileOpenDialog(File *pF, char *szExt, char *szSpec, WORD wAttr)
    : FileDialog(pF, szExt, szSpec, wAttr)
{
}

//
// regDefMsgs registers File Open menu dialog box callback methods.
//
void
FileOpenDialog::regDefMsgs(Handler *pHand)
{
    static CB CmdCbList[] = {
        { IDOK, (CBPROC) &FileOpenDialog::idOk },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regCmdCb(CmdCbList, FALSE);

    // Register base class' default messages
    FileDialog::regDefMsgs(pHand);
}

//
// Try opening the file name in the edit window, or getting a new file list
// if the name has global file characters.
//
LRESULT
FileOpenDialog::idOk(UINT, WPARAM, LPARAM)
{
    Edit fnEdit(hwnd(), IDC_FNAME);
    fnEdit.getText(szFileName, 80);

    short nEditLen = lstrlen(szFileName);
    char cLastChar = *AnsiPrev(szFileName, szFileName + nEditLen);
    if (cLastChar == '\\' || cLastChar == ':')
        lstrcat(szFileName, szFileSpec);

    // If file name has globals, get new directory list
    if (lstrchr(szFileName, '*') || lstrchr(szFileName, '?')) {
        if (updateLBs(szFileName)) {
            lstrcpy(szFileSpec, szFileName);
            fnEdit.setText(szFileSpec);
        } else
            MessageBeep(0);
        return TRUE;
    }

    // Update controls to reflect file selected
    lstrcat(lstrcat(szFileName, "\\"), szFileSpec);
    if (updateLBs(szFileName)) {
        lstrcpy(szFileSpec, szFileName);
        fnEdit.setText(szFileSpec);
    }
    szFileName[nEditLen] = '\0';

    // Try opening the selected file, adding an extension if necessary
    if (!pFile->open(szFileName, OF_READ | OF_EXIST)) {
        lstrcat(szFileName, szDefExt);
        if (!pFile->open(szFileName, OF_READ | OF_EXIST)) {
            MessageBeep(0);
            return TRUE;
        }
    }
    lstrcpy(szFileName, AnsiNext(lstrrchr((LPSTR)pFile->szPathName, '\\')));
    OemToAnsi(szFileName, szFileName);

    EndDialog(hwnd(), TRUE);
    return TRUE;
}


////////////////////////////////////////////
// METHODS FOR FILE SAVE DIALOG BOX CLASS //
////////////////////////////////////////////

//
// Default constructor just calls Dialog constructor.
//
FileSaveDialog::FileSaveDialog(File *pF, char *szExt, char *szSpec)
    : FileDialog(pF, szExt, szSpec, 0)
{
}

//
// regDefMsgs registers File Save menu dialog box callback methods.
//
void
FileSaveDialog::regDefMsgs(Handler *pHand)
{
    static CB CmdCbList[] = {
        { IDOK, (CBPROC) &FileSaveDialog::idOk },
        { 0, NULL }
    };

    if (!pHand) return;
    pHand->regCmdCb(CmdCbList, FALSE);

    // Register base class' default messages
    FileDialog::regDefMsgs(pHand);
}

//
//
LRESULT
FileSaveDialog::idOk(UINT, WPARAM, LPARAM)
{
    Edit fnEdit(hwnd(), IDC_FNAME);
    fnEdit.getText(szFileName, 80);

    short nEditLen = lstrlen(szFileName);
    char cLastChar = *AnsiPrev(szFileName, szFileName + nEditLen);
    if (cLastChar == '\\' || cLastChar == ':')
        lstrcat(szFileName, szFileSpec);

    // If file name has globals, get new directory list
    if (lstrchr(szFileName, '*') || lstrchr(szFileName, '?')) {
        if (updateLBs(szFileName)) {
            lstrcpy(szFileSpec, szFileName);
            fnEdit.setText(szFileSpec);
        } else
            MessageBeep(0);
        return TRUE;
    }

    if (!pFile->open(szFileName, OF_PARSE)) {
        MessageBeep(0);
        return TRUE;
    }

    if (!lstrchr(AnsiNext(lstrrchr((LPSTR)pFile->szPathName, '\\')), '.'))
        lstrcat(szFileName, szDefExt);

    if (pFile->open(szFileName, OF_WRITE | OF_EXIST))
        wStatus = 1;
    else if (pFile->open(szFileName, OF_CREATE | OF_EXIST)) {
        wStatus = 0;        // file is created if it did not already exist
        pFile->unlink();    // delete the file
    }
    else {
        MessageBeep(0);
        return TRUE;
    }

    char path[MAXPATH];
    lstrcpy(path, (LPSTR)pFile->szPathName);
    LPSTR lpFilePart = AnsiNext(lstrrchr(path, '\\'));
    OemToAnsi(lpFilePart, szFileName);
    lpFilePart = AnsiPrev((LPSTR)path, lpFilePart);
    *lpFilePart = 0;
#if (0 MSC(+1))         // Microsoft C++
    _chdir(path);
#elif (0 BCC(+1))       // Borland C++
    chdir(path);
#endif

    EndDialog(hwnd(), TRUE);
    return TRUE;
}


///////////////////////
// UTILITY FUNCTIONS //
///////////////////////

LPSTR
lstrchr(LPSTR str, char ch)
{
    while (*str) {
        if (ch == *str)
            return str;
        str = AnsiNext(str);
    }
    return NULL;
}

LPSTR
lstrrchr(LPSTR str, char ch)
{
    for (LPSTR strl = str + lstrlen(str);
        strl > str;
        strl = AnsiPrev(str, strl))
    {
        if (ch == *strl)
            return strl;
    }
    return NULL;
}
