//
// LIST.CPP     Implementation of List class methods.
//
// Created:     Bruce Clift  8/20/91
//              Greg Voss    5/10/93   adapt for MSC 7.0
//

#include <windows.h>
#pragma hdrstop
#include <memory.h>

#include "wfdefs.hpp"
#if (0 MSC(+1))         // Microsoft C++
#include <string.h>     // for movmem (found in mem.h or memory.h for BCC)
#endif

#include "list.hpp"


//
// The default constructor simply clears the instance variables.
// 
List::List() : pItem(NULL), cItems(0)
{
}

//
// The destructor frees the list memory and sets the list pointer and the
// count to zero to flag that the list is now invalid.
//
void
List::destroy(void)
{
    if (pItem) {
        LocalFree((HLOCAL) pItem);
        pItem = NULL;
    }
    cItems = 0;
}


//
// insert an item in a List.
//  INPUT: wIndex is the index of the item to add.  dwData is the data
// associated with the index.  If bDestructive is TRUE (default), the item
// replaces an existing item in the list.
//  RETURNS: TRUE if item added or replaced, FALSE if no memory.
//
BOOL
List::insert(WORD wIndex, DWORD dwData, BOOL bDestructive)
{
    // For first item, allocate memory for list (off local heap)
    if (!pItem) {
        pItem = (NPITEM) LocalAlloc(LPTR, sizeof(ITEM));
        if (!pItem) return FALSE;
        pItem[0].wIndex = wIndex;
        pItem[0].dwData = dwData;
        cItems = 1;
        return TRUE;
    }

    // Search for index in list
    DWORD dw = find(wIndex);
    int nPos = LOWORD(dw);
    if (HIWORD(dw)) {                   // If index already in list,
        if (!bDestructive) return FALSE;
        pItem[nPos].dwData = dwData;    // replace data only if asked
        return TRUE;
    }

    // Make room for another callback in list
    int size = (cItems + 1) * sizeof(ITEM);
    HLOCAL hNew = LocalReAlloc((HLOCAL) pItem, size, LMEM_MOVEABLE);
    if (!hNew) return FALSE;
    pItem = (NPITEM) hNew;

    // Insert new index & callback into list
#if (0 MSC(+1))         // Microsoft C++
    // memmove is ANSI C; use for both MSC and BCC
    memmove(&pItem[nPos+1], &pItem[nPos], (cItems++ - nPos) * sizeof(ITEM));
#elif (0 BCC(+1))      // Borland C++
    movmem(&pItem[nPos], &pItem[nPos+1], (cItems++ - nPos) * sizeof(ITEM));
#else
#error BCC(x) or MSC(x) must be defined.  See wfdefs.hpp.
#endif
    pItem[nPos].wIndex = wIndex;
    pItem[nPos].dwData = dwData;
    return TRUE;
}


//
// remove an item from a List.
//  INPUT: wIndex is the index of the item to remove.
//  RETURNS: TRUE if item removed, FALSE if index not found.
//
BOOL
List::remove(WORD wIndex)
{
    // Search for index in list
    DWORD dw = find(wIndex);
    if (!HIWORD(dw)) return FALSE;
    int nPos = LOWORD(dw);

    // If list is now empty, free array and set pointer to NULL
    if (!--cItems) {
        LocalFree((HLOCAL) pItem);
        pItem = NULL;
        return TRUE;
    }

    // Remove item from list and shrink the array to new size (ignore error)
  //movmem(&pItem[nPos+1], &pItem[nPos], (cItems - nPos) * sizeof(ITEM));
#if (0 MSC(+1))         // Microsoft C++
    // memmove is ANSI C; use for both MSC and BCC
    memmove(&pItem[nPos], &pItem[nPos+1], (cItems - nPos) * sizeof(ITEM));
#elif (0 BCC(+1))      // Borland C++
    movmem(&pItem[nPos+1], &pItem[nPos], (cItems - nPos) * sizeof(ITEM));
#else
#error BCC(x) or MSC(x) must be defined.  See wfdefs.hpp.
#endif
    int size = cItems * sizeof(ITEM);
    HLOCAL hNew = LocalReAlloc((HLOCAL) pItem, size, LMEM_MOVEABLE);
    if (hNew) pItem = (NPITEM) hNew;
    return TRUE;
}

//
// item is the public function which finds an index in a List.
//  INPUT: wIndex is the index to find.
//  RETURNS: the data associated with the index, or 0 if not found.
//
DWORD
List::item(WORD wIndex)
{
    DWORD dw = find(wIndex);
    int nPos = LOWORD(dw);
    if (HIWORD(dw))                 // If index found
        return pItem[nPos].dwData;  // return associated data
    return 0L;                      // return 0 for failure
}

//
// find performs a binary search to find an item in a List.
//  INPUT: wIndex is the index to find.
//  RETURNS: a double-word, with non-zero in the high word if the item was
// found, and the position of the item in the low word (or if not found, where
// the item should be inserted). 
//
DWORD NEAR
List::find(WORD wIndex)
{
    if (!cItems || pItem == NULL)       // Empty list
        return 0L;

    int top=0, bot=cItems, mid, cmp;
    while (1) {
        mid = (top + bot) >> 1;             // Get midpoint of list
        cmp = pItem[mid].wIndex - wIndex;   // Compare indexes
        if (cmp == 0) return MAKELONG(mid, 1); // return success if match found
        if (bot - top < 2) break;           // No more list to divide?
        if (cmp > 0)    bot = mid;              // Cut list in half
        else        top = mid;
    }
    if (cmp < 0) mid++;                 // Point to correct insertion point
    return MAKELONG(mid, 0);            // Return insertion point & failure
}
