//
// MDI.CPP      Creation methods for MDI classes.
//
// Created:     Bruce Clift  10/21/91
//

#include <windows.h>
#pragma hdrstop

#include "mdi.hpp"
#include "window.hpp"


//////////////////////////////////////////////
// FRAME AND CLIENT WINDOW CREATION METHODS //
//////////////////////////////////////////////

//
// Default constructor simply clears instance variables.
//
MDIFrame::MDIFrame() : Window(), _hwClient(NULL)
{}


//
// Create the client window.
//
BOOL
MDIFrame::setChildren(void)
{
    return (createClient(getWord(GWW_HINSTANCE), hwnd()) != NULL);
}

//
// This virtual method creates the MDI client window.  It is called
// automatically by the MDIFrame create method above.
//
HWND
MDIFrame::createClient(HINSTANCE hInst, HWND hwFrame)
{
    // Get menu information from derived Frame class
    CLIENTCREATESTRUCT ccs;
    ccs.hWindowMenu = getWindowMenu();
    ccs.idFirstChild = getFirstChild();

    // Create the MDI client window
    _hwClient = CreateWindow("MDICLIENT", NULL, getClientStyle(), 0, 0, 0, 0,
        hwFrame, NULL, hInst, (LPSTR) (LPCLIENTCREATESTRUCT) &ccs);

    return _hwClient;
}

//
// Override this method to change the style of the client window.
//
long
MDIFrame::getClientStyle(void)
{
    return (WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE);
}


///////////////////////////////////
// CHILD WINDOW CREATION METHODS //
///////////////////////////////////

//
// Default constructor simply clears instance variables.
//
MDIChild::MDIChild() : Window(), pFrame(NULL)
{}


//
// Child creation method first saves a pointer to the parent MDIFrame
// object. It then fills a MDICREATESTRUCT to hold creation
// parameters.  A pointer to this object is saved in Window's static
// data member pNewWin so that the winProcReg callback function will
// know what object the new window handle is for.  Finally a message
// is sent to the client window to create this child window.
//
HWND
MDIChild::create(LPCSTR cls, LPCSTR title, HINSTANCE hInst,
    LONG lStyle, HWND hwCli, int xx, int yy, int cxx, int cyy, LONG lp)
{
    // Save pointer back to frame window in data member
    pFrame = (MDIFrame *) findWinHnd(GetParent(hwCli));

    // Initialize Handler pointer if not already done
    if (!_pHand)
        setHandler(getHandler());

    // Put creation parameters in MDI child create struct
    MDICREATESTRUCT mcs;
    mcs.szClass = cls;
    mcs.szTitle = title;
    mcs.hOwner = hInst;
    mcs.x = xx;
    mcs.y = yy;
    mcs.cx = cxx;
    mcs.cy = cyy;
    mcs.style = lStyle;
    mcs.lParam = lp;

    // Save ptr to this object in Window static data member
    pNewWin = this;

    // Create the child window
    HWND hw = (HWND) SendMessage(hwCli, WM_MDICREATE, 0, (LONG) &mcs);

    // If the frame is using a status bar, register status bar messages
    StatusBar *pSB = pFrame->getStatusBar();
    if (pSB) regSBMsgs(_pHand);

    return hw;
}
