######################################################################
## MSC 7.0
## C++ makefile for
## DiskTutor class library
######################################################################
## Usage Notes:
## To build dt.lib:
##     nmake /fmsc70.mak
##          or
##     nmake /fmsc70.mak all
##
## To install dt.lib and *.h files:
##     nmake /fmsc70.mak install
##
## To clean intermediate *.obj files:
##     nmake /fmsc70.mak clean
######################################################################


######################################################################
CCDEBUG = /Od /Zi
MODEL = S       # C small memory model
USR_INCLUDE_DIR =   ..\include
USR_LIB_DIR     =   ..\lib
##I = c:\c700\include   ## Environment variables used instead

CC = cl
CFLAGS = $(CCDEBUG) /A$(MODEL) /Gs /I$(USR_INCLUDE_DIR)
USRLIB=wf$(MODEL).lib
######################################################################
## Meaning of CFLAGS for MSC 7.0 Compiler
## /AS      small memory model
## /Gs      turn off run-time stack checking
## /Od      turn off all optimizations (use with /Zi for debugging)
## /Zi      include symbolic debug information in obj files
## /Ipath   dir indicates additional path to search for #include
######################################################################

######################################################################
## Meaning of LFLAGS for MSC 7.0 Linker
## /CO      include symbolic debug information in exe files
######################################################################

# Make the small model by default
all : wf$(MODEL).lib

# Include standard definitions
!include "..\defs_msc.mak"

# Expand inline methods for library
.cpp.obj :
    $(CC) $(CCOPTS) $<
#    $(CC) $(CCOPTS) -H=wf.sym {$& }


#
# LIBRARY OBJECT TARGETS
#
WINO=window.obj wincb.obj winsb.obj wincreat.obj wincre1.obj regwincl.obj rwccls.obj rwcwin.obj
HANDO=handler.obj hndunreg.obj subhand.obj sprhand.obj wpreg.obj wpstd.obj
DLGO=dialog.obj dlgctl.obj modal.obj modeless.obj ribbon.obj ownerdraw.obj
MDIO=mdi.obj mdicb.obj mdivert.obj
RESO=cursor.obj icon.obj menu.obj resource.obj stringre.obj
GDIO=devconte.obj bitmap.obj brush.obj font.obj metafile.obj palette.obj pen.obj region.obj
MISCO=app.obj appmsg.obj list.obj file.obj filedlg.obj profile.obj statusba.obj
MSGO=msgloop1.obj msgloop2.obj msgloop3.obj msgloop4.obj msgloop5.obj msgloop6.obj msgloop7.obj msgloop8.obj

OBJS=$(WINO) $(HANDO) $(DLGO) $(MDIO) $(RESO) $(GDIO) $(MISCO) $(MSGO)

window : $(WINO)
handler : $(HANDO)
dialog : $(DLGO)
mdi : $(MDIO)
resource : $(RESO)
gdi : $(GDIO)
misc : $(MISCO)
msgs : $(MSGO)


#
# WINDOWFRAME LIBRARY
#

$(USRLIB) : $(OBJS)
    @del $@
    lib $(USRLIB) @<<lib.rsp
+window +wincb +winsb +wincreat +wincre1 +regwincl +rwccls +rwcwin &
+handler +hndunreg +subhand +sprhand +wpreg +wpstd &
+dialog +dlgctl +modal +modeless +ribbon +ownerdra &
+mdi +mdicb +mdivert &
+cursor +icon +menu +resource +stringre &
+devconte +bitmap +brush +font +metafile +palette +pen +region &
+app +appmsg +list +file +filedlg +profile +statusba &
+msgloop1 +msgloop2 +msgloop3 +msgloop4 &
+msgloop5 +msgloop6 +msgloop7 +msgloop8, $*.lst
<<KEEP

clean:
    del *.obj
    del *.map
    del *.sym

