//
// OWNERDRA.CPP Implementation of owner-draw buttons.
//
// Created:     Bruce Clift  8/24/92
//

#include <windows.h>
#pragma hdrstop

#include "ownerdra.hpp"
#include "handler.hpp"
#include "rect.hpp"
#include "devconte.hpp"
#include "bitmap.hpp"


/////////////////////////////////////////////
// DEFAULT MESSAGES FOR OWNER-DRAW BUTTONS //
/////////////////////////////////////////////

//
// drawItem should be called when the parent window receives
// a WM_DRAWITEM message.
//
void
OwnerDrawButton::drawItem(LPDRAWITEMSTRUCT lpDIS)
{
    if (lpDIS->itemAction & ODA_FOCUS)
        focus(lpDIS);
    if (lpDIS->itemAction & ODA_SELECT)
        select(lpDIS);
    if (lpDIS->itemAction & ODA_DRAWENTIRE)
        drawEntire(lpDIS);
}

//
// focus is called when the focus of the button has changed.
//
void
OwnerDrawButton::focus(LPDRAWITEMSTRUCT)
{
}

//
// select is called when the selection status of the button has changed.
//
void
OwnerDrawButton::select(LPDRAWITEMSTRUCT lpDIS)
{
    if (lpDIS->itemState & ODS_SELECTED) {
        Rect r;
        *(RECT *)&r = lpDIS->rcItem;
        BitBlt(lpDIS->hDC, r.left, r.top, r.width(), r.height(),
            lpDIS->hDC, r.left, r.top, NOTSRCCOPY);
    }
    else
        drawEntire(lpDIS);
}

//
// drawEntire is called when the entire button must be drawn.
//
void
OwnerDrawButton::drawEntire(LPDRAWITEMSTRUCT lpDIS)
{
    if (!(lpDIS->itemState & ODS_SELECTED)) {
        if (pBitmap1)
            drawBitmap(lpDIS, pBitmap1);
    } else if (pBitmap2)
        drawBitmap(lpDIS, pBitmap2);
}

//
// Helper function to stretch the button's bitmap onto the target
// device context.
//
void
OwnerDrawButton::drawBitmap(LPDRAWITEMSTRUCT lpDIS, Bitmap *pBmp)
{
    DevContext dc;
    dc.createCompatible(lpDIS->hDC);
    HBITMAP hbmpOld = pBmp->select(dc);
    Rect r;
    *(RECT *)&r = lpDIS->rcItem;
    StretchBlt(lpDIS->hDC, r.left, r.top, r.width(), r.height(),
        dc, 0, 0, pBmp->bmWidth, pBmp->bmHeight, SRCCOPY);
    SelectObject(dc, hbmpOld);
}
