//
// REGWINCL.CPP RegWinCls C++ class implementation.
//
// Created:     Bruce Clift  10/12/91
//

#include <windows.h>
#pragma hdrstop

#include "regwincl.hpp"
#include "handler.hpp"

#define SEGNULL(p) (!((DWORD) (p) & 0xffff0000UL))


//
// Constructor to allow user to set the values as needed.
// A Handler object is used instead of a window procedure.
//
RegWinCls::RegWinCls(LPCSTR cn, HINSTANCE hInst, Handler *pHand,
        UINT s, LPCSTR mn, HICON hi, HCURSOR hc, HBRUSH hbr,
        int ce, int we)
{
    // Flag that there are no strings to free yet
    lpszClassName = lpszMenuName = NULL;
    className(cn);          // Copy in class and menu names
    menuName(mn);

    // Some WNDCLASS fields use default flags to call API functions now
    if (hi == DEF_HICON)
            hIcon = LoadIcon(NULL, IDI_APPLICATION);
    else    hIcon = hi;
    if (hc == DEF_HCURSOR)
            hCursor = LoadCursor(NULL, IDC_ARROW);
    else    hCursor = hc;

    // Set other WNDCLASS fields
    hInstance = hInst;
    style = s;
    hbrBackground = hbr;
    cbClsExtra = ce;
    cbWndExtra = we;

    // Messages will be processed using Handler member function
    pNewHandler(pHand);
}

//
// Destructor frees strings.
//
RegWinCls::~RegWinCls()
{
    className(NULL);
    menuName(NULL);
}


//
// The pNewHandler method is used to set the static data member _pNewHandler.
// If the new value is non-NULL, this method also sets the Windows callback
// window procedure to winProcReg (used by all windows registered using the
// RegWinCls class), and makes space for a pointer to a Handler object.  
//
void
RegWinCls::pNewHandler(Handler *pHand)
{
    if ((_pNewHandler=pHand) != NULL) {
        // All classes use winProcReg for Windows' callback
        lpfnWndProc = (WNDPROC) winProcReg;
        cbClsExtra += sizeof(int *);
    }
}



//
// The className and menuName methods with a string parameter first free any
// pre-existing string, then allocate space on the local heap for the class and
// menu name strings.  Because the application's data segment may have moved
// since original allocation, just the offset portion is used for freeing.
//
void
RegWinCls::className(LPCSTR lpStr)
{
    if (lpszClassName) {            // Just use offset portion for freeing
        LocalFree((HLOCAL) LOWORD((LONG) lpszClassName));
        lpszClassName = NULL;
    }

    if (lpStr && *lpStr) {      // Allocate space and copy in string
        lpszClassName = (LPSTR) (PSTR) LocalAlloc(LPTR, lstrlen(lpStr) + 1);
        lstrcpy((LPSTR) lpszClassName, lpStr);
    }
}

void
RegWinCls::menuName(LPCSTR lpStr)
{
    if (lpszMenuName) {             // Just use offset portion for freeing
        if (!SEGNULL(lpszMenuName))
            LocalFree((HLOCAL) LOWORD((LONG) lpszMenuName));
        lpszMenuName = NULL;
    }

    if (!lpStr) return;
    if (SEGNULL(lpStr)) {           // NULL seg: must be int resource
        lpszMenuName = lpStr;
    } else if (*lpStr) {                // Allocate space and copy in string
        lpszMenuName = (LPSTR) (PSTR) LocalAlloc(LPTR, lstrlen(lpStr) + 1);
        lstrcpy((LPSTR) lpszMenuName, lpStr);
    }
}


//
// regClass first calls the API routine to register a window class.  If windows
// of this new class will be using a Handler object to process messages, it
// then calls setNewHand to save a pointer to the Handler object in the class
// extra bytes, so that all windows of this class will have access to it.
//
BOOL
RegWinCls::regClass(void)
{
    if (!RegisterClass(this))       // Register the window class
        return FALSE;

    // If using Handler for messages, put object ptr in class extra bytes
    if (_pNewHandler) {
        _pNewHandler->setNewHand(className(), hInstance);
        _pNewHandler = NULL;
    }
    return TRUE;
}
