//
// SPRHAND.CPP  Implementation of SuperHandler class for SuperClassing.
//
// Created:     Bruce Clift  9/10/91
//

#include <windows.h>
#pragma hdrstop

#include "sprhand.hpp"
#include "regwincl.hpp"


/////////////////////////
// STATIC DATA MEMBERS //
/////////////////////////

// Indexes (from end) of superclass items in class extra bytes
const int SuperHandler::nBCWinProc  = (sizeof(int *) + sizeof(FARPROC));
const int SuperHandler::nBCClsExtra = (sizeof(int *) + sizeof(FARPROC) + 2);
const int SuperHandler::nBCWndExtra = (sizeof(int *) + sizeof(FARPROC) + 4);

// Extra space needed for superclass in registered class's class extra bytes
const int SuperHandler::cbSprHandClsExtra = (sizeof(FARPROC) + 4);


///////////////////////////////////////////
// IMPLEMENTATION FOR SUPERHANDLER CLASS //
///////////////////////////////////////////

//
// Constructors and destructors.
//
SuperHandler::SuperHandler(PCB pMsgList, PCB pCmdList)
    : Handler(pMsgList, pCmdList),
    cbBCClsExtra(0), cbBCWndExtra(0)
{}

SuperHandler::~SuperHandler()
{}

//
// The superClass method registers a SuperClassed window class, setting the
// class/window extra byte amounts and replacing the window procedure.
//
BOOL
SuperHandler::superClass(RegWinCls *pRWC, int cbClsMore, int cbWndMore)
{
    // Save number of extra bytes used by base window class
    cbBCClsExtra = pRWC->cbClsExtra;
    cbBCWndExtra = pRWC->cbWndExtra;

    // Get extra space for SuperClass' bytes
    pRWC->cbClsExtra += cbClsMore + cbSprHandClsExtra;
    pRWC->cbWndExtra += cbWndMore;

    // Replace base class window procedure with our handler
    defWinProc = (FARPROC) pRWC->lpfnWndProc;
    pRWC->pNewHandler(this);

    // Register the SuperClass (destroy the callback lists on failure)
    if (pRWC->regClass())
        return TRUE;
    delete pMsgs;
    delete pCmds;
    return FALSE;
}


//
// set/getClsExtraBytes override the virtual methods in the Handler class. 
// The set method puts instance variable values (obtained while registering
// the class) into the class extra bytes.  It is called immediately after
// class registration.  The get method gets these values from the extra bytes.
// It is called whenever a new window is created.
//  INPUT: hwnd is a temporary window belonging to the registered class.
//
void
SuperHandler::setClsExtraBytes(HWND hwnd)
{
    // Get total number of bytes in class extra bytes (our info is at end)
    WORD cbClsTotal = GetClassWord(hwnd, GCW_CBCLSEXTRA);

    // Set the class extra bytes to instance variable values
    SetClassLong(hwnd, cbClsTotal - nBCWinProc, (LONG) defWinProc);
    SetClassWord(hwnd, cbClsTotal - nBCClsExtra, cbBCClsExtra);
    SetClassWord(hwnd, cbClsTotal - nBCWndExtra, cbBCWndExtra);
}

void
SuperHandler::getClsExtraBytes(HWND hwnd)
{
    // Get total number of bytes in class extra bytes (our info is at end)
    WORD cbClsTotal = GetClassWord(hwnd, GCW_CBCLSEXTRA);

    // Read instance variable values from the class extra bytes
    defWinProc = (FARPROC) GetClassLong(hwnd, cbClsTotal - nBCWinProc);
    cbBCClsExtra = GetClassWord(hwnd, cbClsTotal - nBCClsExtra);
    cbBCWndExtra = GetClassWord(hwnd, cbClsTotal - nBCWndExtra);
}
