//
// SUBHAND.CPP  Implementation for subclassed message handlers.
//
// Created:     Bruce Clift  10/28/91
//

#include <windows.h>
#pragma hdrstop

#include "subhand.hpp"
#include "window.hpp"


//
// Default constructor initializes callback method lists.
//
SubHandler::SubHandler(PCB pMsgList, PCB pCmdList)
    : Handler(pMsgList, pCmdList), _hwSub(NULL)
{}


//
// Destructor restores original win proc for subclassed window.
//
SubHandler::~SubHandler()
{
    if (_hwSub)
        SetWindowLong(_hwSub, GWL_WNDPROC, (LONG) defWinProc);
}


//
// The central method in the SubHandler routine subclasses a single existing
// window.  This allows the filtering of certain messages while keeping most
// behavior of an existing window class.
//
void
SubHandler::subClass(Window *pWin, HINSTANCE hInst)
{
    // Save old window procedure to use as default,
    // while setting new WinProc to be WindowFrame's winProcStd.
    // Also, save the window handle of the subclassed window
    // in an instance variable (_hwSub).
    defWinProc = (FARPROC) SetWindowLong(_hwSub = pWin->hwnd(),
        GWL_WNDPROC, (LONG) getWinProcStd(hInst));

    // Set Handler ptr for winProcStd to find register default messages
    pWin->setHandler(this);
    pWin->regDefMsgs(this);

    pWin->addWinHnd(_hwSub);    // Add window handle to list
}

