//
// WINCREAT.CPP Implementation of WinCreate class methods.
//
// Created:     Bruce Clift  10/20/91
//

#include <windows.h>
#pragma hdrstop

#include "wincreat.hpp"


//
// Constructor fills CREATESTRUCT fields with parameters, and allocates space
// for strings off the local heap.
//
WinCreate::WinCreate(LPCSTR cls, LPCSTR name, HINSTANCE hInst, long lStyle,
    HMENU hMnu, HWND hwParent, int xx, int yy, int cxx, int cyy,
    LPSTR lpParam, long lExStyle)
{
    // Flag that there are no strings to free yet
    lpszClass = lpszName = NULL;
    className(cls);         // Copy in class name and window text
    winName(name);

    // Set other CREATESTRUCT fields
    hInstance = hInst;
    style = lStyle;
    hMenu = hMnu;
    hwndParent = hwParent;
    x = xx;
    y = yy;
    cx = cxx;
    cy = cyy;
    lpCreateParams = lpParam;
    dwExStyle = lExStyle;
}


//
// Destructor frees string memory.
//
WinCreate::~WinCreate()
{
    className(NULL);
    winName(NULL);
}


//
// The className and winName methods with a string parameter first free any
// pre-existing string, then allocate space on the local heap for the class
// name and window text strings.  Because the application's data segment may
// have moved since original allocation, just the offset portion is used for
// freeing. 
//
void
WinCreate::className(LPCSTR lpStr)
{
    if (lpszClass) {                // Just use offset portion for freeing
        LocalFree((HLOCAL) LOWORD((LONG) lpszClass));
        lpszClass = NULL;
    }
    if (lpStr && *lpStr) {      // Allocate space and copy in string
        lpszClass = (LPSTR) (PSTR) LocalAlloc(LPTR, lstrlen(lpStr) + 1);
        lstrcpy((LPSTR) lpszClass, lpStr);
    }
}

void
WinCreate::winName(LPCSTR lpStr)
{
    if (lpszName) {             // Just use offset portion for freeing
        LocalFree((HLOCAL) LOWORD((LONG) lpszName));
        lpszName = NULL;
    }
    if (lpStr && *lpStr) {      // Allocate space and copy in string
        lpszName = (LPSTR) (PSTR) LocalAlloc(LPTR, lstrlen(lpStr) + 1);
        lstrcpy((LPSTR) lpszName, lpStr);
    }
}
