//
// WPSTD.CPP    winProcStd handles messages for standard Windows classes.
//
// Created:     Bruce Clift  9/18/91
//

#include <windows.h>
#pragma hdrstop

#include "handler.hpp"
#include "window.hpp"
#include "dialog.hpp"


//
// Gets a procedure instance for winProcStd.  Makes it the first time
// through, and thereafter returns the pointer.
//
FARPROC
Handler::getWinProcStd(HINSTANCE hInst)
{
    if (!lpfnWPStd)
        lpfnWPStd = MakeProcInstance((FARPROC) winProcStd, hInst);
    return lpfnWPStd;
}

//
// winProcStd is the exported window procedure which Windows calls for
// windows belonging to standard Windows classes (including dialog boxes).
// It first gets a pointer to the Window object associated with the given
// window handle.  For newly created windows this pointer has been saved in
// Window's static member pNewWin.  This pointer and the associated window
// handle are added to the list in Window.  A pointer to the Handler object
// is then obtained from the window object.  Finally, the dispatch method
// is called dispatch the message. 
//
extern "C" LRESULT FAR PASCAL EXPORT
winProcStd(HWND hwnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
{
    // Get pointer to Window object associated with this window handle
    Window *pWin;
    if (Window::pNewWin &&
        (wMsg==WM_GETMINMAXINFO || wMsg==WM_NCCREATE || wMsg==WM_INITDIALOG))
            pWin = Window::newWinHnd(hwnd);     // Add new windows to list
    else    pWin = Window::findWinHnd(hwnd);    // Get old windows from list

    // Once WM_INITDIALOG is sent, the child control list is no longer needed
    if (wMsg == WM_INITDIALOG && pWin)
        ((Dialog *) pWin)->destroyChildren();

    // Get Handler object pointer from the Window object
    Handler *pHand = (pWin ? pWin->getHandler() : NULL);

    // Make sure we have all the pointers we need
    if (!pHand)
        return CallWindowProc((FARPROC) DefWindowProc,
            hwnd, wMsg, wParam, lParam);

    return pWin->dispatch(wMsg, wParam, lParam);
}
